#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.20 06:00:00                  #
# ================================================== #

from PySide6.QtGui import QAction

from pygpt_net.utils import trans


class Audio:
    def __init__(self, window=None):
        """
        Menu setup

        :param window: Window instance
        """
        self.window = window

    def setup(self):
        """Setup audio menu"""
        self.window.ui.menu['audio.output'] = QAction(
            trans("menu.audio.output"),
            self.window,
            checkable=True,
        )
        self.window.ui.menu['audio.input'] = QAction(
            trans("menu.audio.input"),
            self.window,
            checkable=True,
        )

        self.window.ui.menu['audio.output'].triggered.connect(
            lambda: self.window.controller.plugins.toggle('audio_output')
        )
        self.window.ui.menu['audio.input'].triggered.connect(
            lambda: self.window.controller.plugins.toggle('audio_input')
        )

        self.window.ui.menu['menu.audio'] = self.window.menuBar().addMenu(trans("menu.audio"))
        self.window.ui.menu['menu.audio'].addAction(self.window.ui.menu['audio.input'])
        self.window.ui.menu['menu.audio'].addAction(self.window.ui.menu['audio.output'])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.09 23:00:00                  #
# ================================================== #

import os
import shutil

from datetime import datetime
from pathlib import PurePath

from PySide6.QtCore import QUrl

from .actions import Actions
from .editor import Editor
from .types import Types
from .url import Url

class Filesystem:
    def __init__(self, window=None):
        """
        Filesystem core

        :param window: Window instance
        """
        self.window = window
        self.actions = Actions(window)
        self.editor = Editor(window)
        self.types = Types(window)
        self.url = Url(window)
        self.workdir_placeholder = "%workdir%"
        self.styles = [
            "style.css",
            "style.dark.css",
            "style.light.css",
            "markdown.css",
            "markdown.dark.css",
            "markdown.light.css",
            "fix_windows.css",
        ]

    def install(self):
        """Install provider data"""
        # data directory
        data_dir = self.window.core.config.get_user_dir('data')
        if not os.path.exists(data_dir):
            os.makedirs(data_dir, exist_ok=True)

        # upload directory
        upload_dir = self.window.core.config.get_user_dir('upload')
        if not os.path.exists(upload_dir):
            os.makedirs(upload_dir, exist_ok=True)

        self.install_css()  # install custom css styles

    def install_css(self, force: bool = False):
        """
        Install custom css styles

        :param force: force install
        """
        css_dir = os.path.join(self.window.core.config.path, 'css')
        if not os.path.exists(css_dir):
            os.mkdir(css_dir)

        src_dir = os.path.join(self.window.core.config.get_app_path(), 'data', 'css')
        dst_dir = os.path.join(self.window.core.config.path, 'css')

        try:
            for style in self.styles:
                src = os.path.join(src_dir, style)
                dst = os.path.join(dst_dir, style)
                if (not os.path.exists(dst) or force) and os.path.exists(src):
                    shutil.copyfile(src, dst)
        except Exception as e:
            print("Error while installing css files: ", e)

    def backup_custom_css(self):
        """Backup user custom css styles"""
        css_dir = os.path.join(self.window.core.config.path, 'css')
        backup_file_extension = '.backup'
        for style in self.styles:
            src = os.path.join(css_dir, style)
            dst = os.path.join(css_dir, style + backup_file_extension)
            if os.path.exists(src):
                shutil.copyfile(src, dst)

    def make_local(self, path: str) -> str:
        """
        Make local placeholder path

        :param path: path to prepare
        :return: local path with working dir placeholder
        """
        return path.replace(
            self.window.core.config.get_user_path(),
            self.workdir_placeholder
        )

    def make_local_list(self, paths: list) -> list:
        """
        Make local placeholder paths

        :param paths: list with paths to prepare
        :return: local paths with working dir placeholder
        """
        return [self.make_local(path) for path in paths]

    def make_local_list_img(self, paths: list) -> list:
        """
        Make local placeholder paths for images

        :param paths: list with paths to prepare
        :return: local paths with working dir placeholder
        """
        img_ext = self.types.get_img_ext()
        result = []
        for path in paths:
            if path.endswith(tuple(img_ext)):
                result.append(self.make_local(path))
        return result

    def get_url(self, url) -> QUrl:
        """
        Make current OS-specific URL to open file or directory

        :param url: URL to prepare
        :return: URL to open file or directory
        """
        if self.window.core.platforms.is_windows():
            if not url.startswith('file:///'):
                url = 'file:///' + url
            return QUrl(url, QUrl.TolerantMode)
        else:
            return QUrl.fromLocalFile(url)

    def get_path(self, path) -> str:
        """
        Prepare current OS-specific path from given path

        :param path: path to prepare
        :return: prepared OS-specific path
        """
        parts = PurePath(path).parts
        if len(parts) > 1:
            return os.path.join(*parts)  # rebuild OS directory separators
        return path

    def to_workdir(self, path) -> str:
        """
        Replace user path with current workdir

        :param path: path to fix
        :return: path with replaced user workdir
        """
        path = self.get_path(path)
        work_dir = self.window.core.config.get_user_path()  # current OS app workdir

        # try to find %workdir% placeholder in path
        if self.workdir_placeholder in path:
            return path.replace(
                self.workdir_placeholder,
                work_dir
            )

        # try to find workdir in path: old versions compatibility, < 2.0.113
        if work_dir.endswith('.config/pygpt-net'):
            work_dir = work_dir.rsplit('/.config/pygpt-net', 1)[0]
        elif work_dir.endswith('.config\\pygpt-net'):
            work_dir = work_dir.rsplit('\\.config\\pygpt-net', 1)[0]

        if self.window.core.platforms.is_windows():
            dir_index = path.find('\\.config\\pygpt-net\\') + 1
        else:
            dir_index = path.find('/.config/pygpt-net/') + 1

        parts = path[dir_index:]
        return os.path.join(work_dir, parts)

    def extract_local_url(self, path) -> (str, str):
        """
        Extract local url and path from url

        :param path: local path or url
        :return: url, path
        """
        if not self.is_schema(path):
            path = self.to_workdir(path)

        prefix = ''
        if not self.is_schema(path):
            if self.window.core.platforms.is_windows():
                prefix = 'file:///'
            else:
                prefix = 'file://'

        url = prefix + path
        return url, path

    def get_workdir_prefix(self) -> str:
        """
        Get workdir prefix

        :return: workdir prefix
        """
        prefix = ''
        if self.window.core.platforms.is_windows():
            prefix = 'file:///'
        else:
            prefix = 'file://'
        return prefix + self.window.core.config.get_user_path()

    def in_work_dir(self, path: str) -> bool:
        """
        Check if path is in work directory

        :param path: path to file
        :return: True if path is in work directory
        """
        work_dir = self.window.core.config.get_user_dir("data")
        return path.startswith(work_dir)

    def store_upload(self, path: str) -> str:
        """
        Store file in upload directory

        :param path: path to uploading file
        :return: path to stored uploaded file
        """
        upload_dir = self.window.core.config.get_user_dir("upload")
        file_name = os.path.basename(path)
        upload_path = os.path.join(upload_dir, file_name)
        # if file exists, add datetime prefix
        if os.path.exists(upload_path):
            upload_path = os.path.join(upload_dir, datetime.now().strftime("%Y-%m-%d_%H-%M-%S_") + file_name)
        shutil.copyfile(path, upload_path)
        return upload_path

    def remove_upload(self, path: str) -> None:
        """
        Delete uploaded file

        :param path: path to uploading file
        """
        upload_dir = self.window.core.config.get_user_dir("upload")
        if path.startswith(upload_dir):
            if os.path.exists(path):
                os.remove(path)

    def is_schema(self, path: str) -> bool:
        """
        Check if path has schema prefix (http, https, file)

        :param path: path to check
        :return: True if path has schema prefix
        """
        return path.startswith('file://') or path.startswith('http://') or path.startswith('https://')

    def sizeof_fmt(self, num: float, suffix='B') -> str:
        """
        Convert numbers to human-readable unit formats.

        :param num: number to convert
        :param suffix: suffix to add
        :return: human-readable format
        """
        for unit in ['', 'K', 'M', 'G', 'T', 'P', 'E', 'Z']:
            if abs(num) < 1024.0:
                return f"{num:3.1f}{unit}{suffix}"
            num /= 1024.0
        return f"{num:.1f}Yi{suffix}"

    def get_directory_size(self, directory : str, human_readable: bool = True) -> str or int:
        """
        Calculate the total size of the given directory

        :param directory: directory path
        :param human_readable: return human-readable format
        :return: total size of the directory
        """
        total_size = 0
        for dirpath, dirnames, filenames in os.walk(directory):
            for f in filenames:
                fp = os.path.join(dirpath, f)
                # skip if symbolic link
                if not os.path.islink(fp):
                    total_size += os.path.getsize(fp)

        if human_readable:
            return self.sizeof_fmt(total_size)
        return total_size

    def get_free_disk_space(self, directory: str, human_readable: bool = True) -> str or int:
        """
        Check free disk space in the given directory

        :param directory: directory path
        :param human_readable: return human-readable format
        :return: free disk space
        """
        total, used, free = shutil.disk_usage(os.path.dirname(directory))
        if human_readable:
            return self.sizeof_fmt(free)
        return free

    def copy_workdir(self, path: str, new_path: str) -> bool:
        """
        Copy working directory

        :param path: current working directory
        :param new_path: new working directory
        :return: True if working directory is copied
        """
        if os.path.isdir(path):
            for item in os.listdir(path):
                s = os.path.join(path, item)
                d = os.path.join(new_path, item)
                if os.path.isdir(s):
                    shutil.copytree(s, d, symlinks=False, ignore=None)
                else:
                    shutil.copy2(s, d)

        # put empty "path.cfg"
        lock_file = os.path.join(new_path, 'path.cfg')
        with open(lock_file, 'w', encoding='utf-8') as f:
            f.write('')

        # remove "profile.json" if exists
        profile_file = os.path.join(new_path, 'profile.json')
        if os.path.exists(profile_file):
            os.remove(profile_file)
        return True

    def clear_workdir(self, path: str) -> bool:
        """
        Clear working directory

        :param path: path to working directory
        :return: True if working directory is cleared
        """
        persist_files = ["app.log", "path.cfg", "profile.json"]
        for item in os.listdir(path):
            item_path = os.path.join(path, item)
            if os.path.isfile(item_path):
                if item not in persist_files:
                    os.remove(item_path)
            else:
                shutil.rmtree(item_path)
        return True

    def is_workdir_in_path(self, path: str) -> bool:
        """
        Check if workdir is in path

        :param path: path to check
        :return: True if workdir is in path
        """
        files = ["config.json", "db.sqlite"]
        for file in files:
            if not os.path.exists(os.path.join(path, file)):
                return False
        return True

    def is_directory_empty(self, path: str) -> bool:
        """
        Check if directory is empty

        :param path: directory path
        :return: True if directory is empty
        """
        return len(os.listdir(path)) == 0




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.09 23:00:00                  #
# ================================================== #

import os

from .capture import Capture
from .common import Common


class Painter:
    def __init__(self, window=None):
        """
        Painter controller

        :param window: Window instance
        """
        self.window = window
        self.capture = Capture(window)
        self.common = Common(window)

    def setup(self):
        """Setup painter"""
        self.restore()
        size = "800x600"  # default size
        if self.window.core.config.has('painter.canvas.size'):
            size = self.window.core.config.get('painter.canvas.size')
        self.common.change_canvas_size(size)
        self.common.restore_brush_settings()

    def open(self, path: str):
        """
        Open image from file

        :param path: path to image
        """
        self.window.ui.painter.open_image(path)
        self.window.ui.status("Image loaded: " + os.path.basename(path))

    def open_external(self, path: str):
        """
        Open image from file (from external)

        :param path: path to image
        """
        self.open(path)
        if not self.is_active():
            self.window.controller.ui.switch_tab('draw')

    def save(self):
        """Store current image"""
        path = os.path.join(self.common.get_capture_dir(), '_current.png')
        self.window.ui.painter.image.save(path)

    def save_all(self):
        """Save all (on exit)"""
        self.save()

    def restore(self):
        """Restore previous image"""
        path = os.path.join(self.common.get_capture_dir(), '_current.png')
        if os.path.exists(path):
            self.window.ui.painter.image.load(path)
        else:
            # clear image
            self.window.ui.painter.clear_image()
        self.window.ui.painter.update()

    def is_active(self) -> bool:
        """
        Check if painter is enabled

        :return: True if painter is current active tab
        """
        return self.window.controller.ui.current_tab == self.window.controller.ui.tab_idx['draw']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.25 22:00:00                  #
# ================================================== #


class AssistantsDebug:
    def __init__(self, window=None):
        """
        Assistants debug

        :param window: Window instance
        """
        self.window = window
        self.id = 'assistants'

    def update(self):
        """Update debug window."""
        self.window.core.debug.begin(self.id)

        self.window.core.debug.add(self.id, '(thread) started', str(self.window.controller.assistant.threads.started))
        self.window.core.debug.add(self.id, '(thread) stop', str(self.window.controller.assistant.threads.stop))
        self.window.core.debug.add(self.id, '(thread) run_id', str(self.window.controller.assistant.threads.run_id))
        self.window.core.debug.add(self.id, '(thread) tool_calls', str(self.window.controller.assistant.threads.tool_calls))

        self.window.core.debug.add(
            self.id, 'Options',
            str(self.window.controller.assistant.editor.get_options())
        )

        self.window.core.debug.add(self.id, '----', '')

        # assistants
        assistants = self.window.core.assistants.get_all()
        for key in list(assistants):
            prefix = "[{}] ".format(key)
            assistant = assistants[key]
            data = {
                'id': assistant.id,
                'name': assistant.name,
                'description': assistant.description,
                'model': assistant.model,
                'instructions': assistant.instructions,
                'meta': assistant.meta,
                'tools': assistant.tools,
                'files': assistant.files,
                'tools[function]': assistant.tools['function']
            }
            self.window.core.debug.add(self.id, str(assistant.name), str(data))

        self.window.core.debug.end(self.id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.20 06:00:00                  #
# ================================================== #

from PySide6.QtCore import Slot, Signal

from pygpt_net.plugin.base import BaseWorker, BaseSignals


class WorkerSignals(BaseSignals):
    output = Signal(object, str)
    clear = Signal()


class Worker(BaseWorker):
    def __init__(self, *args, **kwargs):
        super(Worker, self).__init__()
        self.signals = WorkerSignals()
        self.args = args
        self.kwargs = kwargs
        self.plugin = None
        self.cmds = None
        self.ctx = None

    @Slot()
    def run(self):
        responses = []
        for item in self.cmds:
            try:
                response = None
                if item["cmd"] in self.plugin.allowed_cmds and self.plugin.has_cmd(item["cmd"]):

                    if item["cmd"] == "code_execute_file":
                        response = self.cmd_code_execute_file(item)

                    elif item["cmd"] == "code_execute":
                        response = self.cmd_code_execute(item)
                        if "silent" in item:
                            response = None

                    elif item["cmd"] == "code_execute_all":
                        response = self.cmd_code_execute_all(item)
                        if "silent" in item:
                            response = None

                    elif item["cmd"] == "sys_exec":
                        response = self.cmd_sys_exec(item)

                    elif item["cmd"] == "get_python_output":
                        response = self.cmd_get_python_output(item)

                    elif item["cmd"] == "get_python_input":
                        response = self.cmd_get_python_input(item)

                    elif item["cmd"] == "clear_python_output":
                        response = self.cmd_clear_python_output(item)

                    if response:
                        responses.append(response)

            except Exception as e:
                responses.append({
                    "request": {
                        "cmd": item["cmd"],
                    },
                    "result": "Error {}".format(e),
                })
                self.error(e)
                self.log("Error: {}".format(e))

        # send response
        if len(responses) > 0:
            for response in responses:
                self.reply(response)

    def cmd_code_execute_file(self, item: dict) -> dict:
        """
        Execute code command from existing file

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        try:
            if not self.plugin.runner.is_sandbox():
                response = self.plugin.runner.code_execute_file_host(
                    ctx=self.ctx,
                    item=item,
                    request=request,
                )
            else:
                response = self.plugin.runner.code_execute_file_sandbox(
                    ctx=self.ctx,
                    item=item,
                    request=request,
                )
        except Exception as e:
            response = {
                "request": request,
                "result": "Error: {}".format(e),
            }
            self.error(e)
            self.log("Error: {}".format(e))
        return response

    def cmd_code_execute(self, item: dict) -> dict:
        """
        Execute code command

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        try:
            if not self.plugin.runner.is_sandbox():
                response = self.plugin.runner.code_execute_host(
                    ctx=self.ctx,
                    item=item,
                    request=request,
                )
            else:
                response = self.plugin.runner.code_execute_sandbox(
                    ctx=self.ctx,
                    item=item,
                    request=request,
                )
        except Exception as e:
            response = {
                "request": request,
                "result": "Error: {}".format(e),
            }
            self.error(e)
            self.log("Error: {}".format(e))
        return response

    def cmd_code_execute_all(self, item: dict) -> dict:
        """
        Execute all code command

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        try:
            if not self.plugin.runner.is_sandbox():
                response = self.plugin.runner.code_execute_host(
                    ctx=self.ctx,
                    item=item,
                    request=request,
                    all=True,
                )
            else:
                response = self.plugin.runner.code_execute_sandbox(
                    ctx=self.ctx,
                    item=item,
                    request=request,
                    all=True,
                )
        except Exception as e:
            response = {
                "request": request,
                "result": "Error: {}".format(e),
            }
            self.error(e)
            self.log("Error: {}".format(e))
        return response

    def cmd_sys_exec(self, item: dict) -> dict:
        """
        Execute system command

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        try:
            if not self.plugin.runner.is_sandbox():
                response = self.plugin.runner.sys_exec_host(
                    ctx=self.ctx,
                    item=item,
                    request=request,
                )
            else:
                response = self.plugin.runner.sys_exec_sandbox(
                    ctx=self.ctx,
                    item=item,
                    request=request,
                )
        except Exception as e:
            response = {
                "request": request,
                "result": "Error: {}".format(e),
            }
            self.error(e)
            self.log("Error: {}".format(e))
        return response

    def cmd_get_python_output(self, item: dict) -> dict:
        """
        Get python output

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        try:
            output = self.plugin.window.tools.get("interpreter").get_current_output()
            response = {
                "request": request,
                "result": output,
                "context": output,
            }
        except Exception as e:
            response = {
                "request": request,
                "result": "Error: {}".format(e),
            }
            self.error(e)
            self.log("Error: {}".format(e))
        return response

    def cmd_get_python_input(self, item: dict) -> dict:
        """
        Get python input (edit code)

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        try:
            input = self.plugin.window.tools.get("interpreter").get_current_history()
            response = {
                "request": request,
                "result": input,
                "context": input,
            }
        except Exception as e:
            response = {
                "request": request,
                "result": "Error: {}".format(e),
            }
            self.error(e)
            self.log("Error: {}".format(e))
        return response

    def cmd_clear_python_output(self, item: dict) -> dict:
        """
        Clear python output

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        try:
            self.signals.clear.emit()
            response = {
                "request": request,
                "result": "OK",
            }
        except Exception as e:
            response = {
                "request": request,
                "result": "Error: {}".format(e),
            }
            self.error(e)
            self.log("Error: {}".format(e))
        return response

    def prepare_request(self, item) -> dict:
        """
        Prepare request item for result

        :param item: item with parameters
        :return: request item
        """
        return {"cmd": item["cmd"]}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.26 15:00:00                  #
# ================================================== #

from pygpt_net.ui.widget.dialog.base import BaseDialog

class BaseTool:
    def __init__(self, *args, **kwargs):
        """
        Base Tool

        :param window: Window instance
        :param args: arguments
        :param kwargs: keyword arguments
        """
        self.window = None
        self.id = ""

    def setup(self):
        """Setup tool"""
        pass

    def post_setup(self):
        """Post-setup (window), after plugins are loaded"""
        pass

    def on_update(self):
        """On app main loop update"""
        pass

    def on_post_update(self):
        """On app main loop update"""
        pass

    def on_exit(self):
        """On app exit"""
        pass

    def attach(self, window):
        """
        Attach window to plugin

        :param window: Window instance
        """
        self.window = window

    def setup_menu(self) -> list:
        """
        Setup main menu

        :return list with menu actions
        """
        return []

    def setup_dialogs(self):
        """Setup dialogs (static)"""
        pass

    def setup_theme(self):
        """Setup theme"""
        pass

    def get_instance(self, id: str, dialog_id: str = None) -> BaseDialog or None:
        """
        Spawn and return dialog instance

        :param id: dialog instance ID
        :param dialog_id: dialog instance ID
        """
        return None

    def get_lang_mappings(self) -> dict:
        """
        Get language mappings

        :return: dict with language mappings
        """
        return {}


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.25 13:00:00                  #
# ================================================== #

import re

from pygpt_net.controller.presets.editor import Editor
from pygpt_net.utils import trans


class Presets:
    def __init__(self, window=None):
        """
        Presets controller

        :param window: Window instance
        """
        self.window = window
        self.editor = Editor(window)
        self.locked = False

    def setup(self):
        """Setup presets"""
        self.editor.setup()

    def select(self, idx: int):
        """
        Select preset

        :param idx: value of the list (row idx)
        """
        # check if preset change is not locked
        if self.preset_change_locked():
            return
        mode = self.window.core.config.get('mode')
        self.set_by_idx(mode, idx)  # set global and current dict

        # update all layout
        self.window.controller.ui.update()

    def use(self):
        """Copy preset prompt to input"""
        self.window.controller.chat.common.append_to_input(
            self.window.ui.nodes['preset.prompt'].toPlainText()
        )

    def set(self, mode: str, preset_id: str):
        """
        Set preset

        :param mode: mode name
        :param preset_id: preset ID
        """
        if not self.window.core.presets.has(mode, preset_id):
            return False
        self.window.core.config.data['preset'] = preset_id
        if 'current_preset' not in self.window.core.config.data:
            self.window.core.config.data['current_preset'] = {}
        self.window.core.config.data['current_preset'][mode] = preset_id

    def set_by_idx(self, mode: str, idx: int):
        """
        Set preset by index

        :param mode: mode name
        :param idx: preset index
        """
        preset_id = self.window.core.presets.get_by_idx(idx, mode)

        # set preset
        self.window.core.config.set("preset", preset_id)
        if 'current_preset' not in self.window.core.config.data:
            self.window.core.config.data['current_preset'] = {}
        self.window.core.config.data['current_preset'][mode] = preset_id

        # select model
        self.select_model()

    def select_current(self):
        """Select preset by current"""
        mode = self.window.core.config.get('mode')
        preset_id = self.window.core.config.get('preset')
        items = self.window.core.presets.get_by_mode(mode)
        if preset_id in items:
            idx = list(items.keys()).index(preset_id)
            current = self.window.ui.models['preset.presets'].index(idx, 0)
            self.window.ui.nodes['preset.presets'].setCurrentIndex(current)

    def select_default(self):
        """Set default preset"""
        preset_id = self.window.core.config.get('preset')

        # if preset is not set, set default
        if preset_id is None or preset_id == "":
            mode = self.window.core.config.get('mode')

            # set previously selected preset
            current = self.window.core.config.get('current_preset')  # dict of modes, preset per mode
            if mode in current and \
                    current[mode] is not None and \
                    current[mode] != "" and \
                    current[mode] in self.window.core.presets.get_by_mode(mode):
                self.window.core.config.set('preset', current[mode])
            else:
                # or set default preset
                self.window.core.config.set('preset', self.window.core.presets.get_default(mode))

    def update_data(self):
        """Update preset data"""
        preset_id = self.window.core.config.get('preset')
        if preset_id is None or preset_id == "":
            self.reset()  # clear preset fields
            self.window.controller.mode.reset_current()
            return

        if preset_id not in self.window.core.presets.items:
            self.window.core.config.set('preset', "")  # clear preset if not found
            self.reset()  # clear preset fields
            self.window.controller.mode.reset_current()
            return

        # update preset fields
        preset = self.window.core.presets.items[preset_id]
        self.window.ui.nodes['preset.prompt'].setPlainText(preset.prompt)
        # self.window.ui.nodes['preset.ai_name'].setText(preset.ai_name)
        # self.window.ui.nodes['preset.user_name'].setText(preset.user_name)

        # update current data
        self.window.core.config.set('prompt', preset.prompt)
        self.window.core.config.set('ai_name', preset.ai_name)
        self.window.core.config.set('user_name', preset.user_name)

    def update_current(self):
        """Update current mode, model and preset"""
        mode = self.window.core.config.get('mode')

        # if preset chosen, update current config
        preset_id = self.window.core.config.get('preset')
        if preset_id is not None and preset_id != "":
            if preset_id in self.window.core.presets.items:
                preset = self.window.core.presets.items[preset_id]
                self.window.core.config.set('user_name', preset.user_name)
                self.window.core.config.set('ai_name', preset.ai_name)
                self.window.core.config.set('prompt', preset.prompt)
                self.window.core.config.set('temperature', preset.temperature)
                return

        self.window.core.config.set('user_name', None)
        self.window.core.config.set('ai_name', None)
        self.window.core.config.set('temperature', 1.0)

        # set default prompt if mode is chat
        if mode == 'chat':
            self.window.core.config.set('prompt', self.window.core.prompt.get('default'))
        else:
            self.window.core.config.set('prompt', None)

    def get_current_functions(self) -> list or None:
        """
        Get current preset functions

        :return: list of functions
        """
        preset_id = self.window.core.config.get('preset')
        if preset_id is not None and preset_id != "":
            if preset_id in self.window.core.presets.items:
                preset = self.window.core.presets.items[preset_id]
                if preset.has_functions():
                    return preset.get_functions()
        return None

    def from_global(self):
        """Update current preset from global prompt"""
        if self.locked:
            return
        preset_id = self.window.core.config.get('preset')
        if preset_id is not None and preset_id != "":
            if preset_id in self.window.core.presets.items:
                preset = self.window.core.presets.items[preset_id]
                preset.prompt = self.window.core.config.get('prompt')
                self.window.core.presets.save(preset)

    def select_model(self):
        """Select model by current preset"""
        mode = self.window.core.config.get('mode')
        preset_id = self.window.core.config.get('preset')
        if preset_id is not None and preset_id != "":
            if preset_id in self.window.core.presets.items:
                preset = self.window.core.presets.items[preset_id]
                if preset.model is not None and preset.model != "" and preset.model != "_":
                    if preset.model in self.window.core.models.items:
                        if self.window.core.models.has(preset.model) \
                                and self.window.core.models.is_allowed(preset.model, mode):
                            self.window.core.config.set('model', preset.model)
                            self.window.controller.model.set(mode, preset.model)
                            self.window.controller.model.update_list()
                            self.window.controller.model.select_current()

    def refresh(self):
        """Refresh presets"""
        mode = self.window.core.config.get('mode')
        if mode == 'assistant':
            return

        self.select_default()  # if no preset then select previous from current presets
        self.update_current()  # apply preset data to current config
        self.update_data()  # update config and prompt from preset data
        self.window.controller.mode.update_temperature()
        self.update_list()  # update presets list only
        self.select_current()  # select current preset on list

    def update_list(self):
        """Update presets list"""
        mode = self.window.core.config.get('mode')
        items = self.window.core.presets.get_by_mode(mode)
        self.window.ui.toolbox.presets.update(items)

    def reset(self):
        """Reset preset data"""
        self.window.ui.nodes['preset.prompt'].setPlainText("")
        # self.window.ui.nodes['preset.ai_name'].setText("")
        # self.window.ui.nodes['preset.user_name'].setText("")

    def make_filename(self, name: str) -> str:
        """
        Make preset filename from name

        :param name: preset name
        :return: preset filename
        """
        filename = name.lower()
        filename = re.sub(r'[^a-zA-Z0-9_\-\.]', '_', filename)
        return filename

    def duplicate(self, idx: int = None):
        """
        Duplicate preset

        :param idx: preset index (row index)
        """
        if idx is not None:
            mode = self.window.core.config.get('mode')
            preset = self.window.core.presets.get_by_idx(idx, mode)
            if preset is not None and preset != "":
                if preset in self.window.core.presets.items:
                    new_id = self.window.core.presets.duplicate(preset)
                    self.window.core.config.set('preset', new_id)
                    self.refresh()
                    idx = self.window.core.presets.get_idx_by_id(mode, new_id)
                    self.editor.edit(idx)
                    self.window.ui.status(trans('status.preset.duplicated'))

    def clear(self, force: bool = False):
        """
        Clear preset data

        :param force: force clear data
        """
        preset = self.window.core.config.get('preset')

        if not force:
            self.window.ui.dialogs.confirm(
                type='preset_clear', 
                id='', 
                msg=trans('confirm.preset.clear'),
            )
            return

        self.window.core.config.set('prompt', "")
        self.window.core.config.set('ai_name', "")
        self.window.core.config.set('user_name', "")
        self.window.core.config.set('temperature', 1.0)

        if preset is not None and preset != "":
            if preset in self.window.core.presets.items:
                self.window.core.presets.items[preset].ai_name = ""
                self.window.core.presets.items[preset].user_name = ""
                self.window.core.presets.items[preset].prompt = ""
                self.window.core.presets.items[preset].temperature = 1.0
                self.refresh()

        self.window.ui.status(trans('status.preset.cleared'))

        # reload assistant default instructions
        mode = self.window.core.config.get('mode')
        if mode == "assistant":
            self.window.core.assistants.load()

    def delete(self, idx: int = None, force: bool = False):
        """
        Delete preset

        :param idx: preset index (row index)
        :param force: force delete without confirmation
        """
        if idx is not None:
            mode = self.window.core.config.get('mode')
            preset_id = self.window.core.presets.get_by_idx(idx, mode)
            if preset_id is not None and preset_id != "":
                if preset_id in self.window.core.presets.items:
                    # if exists show confirmation dialog
                    if not force:
                        self.window.ui.dialogs.confirm(
                            type='preset_delete', 
                            id=idx, 
                            msg=trans('confirm.preset.delete'),
                        )
                        return

                    if preset_id == self.window.core.config.get('preset'):
                        self.window.core.config.set('preset', None)
                        self.window.ui.nodes['preset.prompt'].setPlainText("")
                    self.window.core.presets.remove(preset_id, True)
                    self.refresh()
                    self.window.ui.status(trans('status.preset.deleted'))

    def restore(self, force: bool = False):
        """
        Restore preset data

        :param force: force restore data
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='preset_restore',
                id='',
                msg=trans('confirm.preset.restore'),
            )
            return
        mode = self.window.core.config.get('mode')
        self.window.core.presets.restore(mode)
        self.refresh()

    def is_current(self, idx: int = None) -> bool:
        """
        Check if preset is current

        :param idx: preset index (row index)
        :return: True if current
        """
        if idx is not None:
            mode = self.window.core.config.get('mode')
            preset_id = self.window.core.presets.get_by_idx(idx, mode)
            if preset_id is not None and preset_id != "":
                if preset_id == "current." + mode:
                    return True
        return False

    def validate_filename(self, value: str) -> str:
        """
        Validate filename

        :param value: filename
        :return: sanitized filename
        """
        # strip not allowed characters
        return re.sub(r'[^\w\s\-\.]', '', value)

    def preset_change_locked(self) -> bool:
        """
        Check if preset change is locked

        :return: True if locked
        """
        # if self.window.controller.chat.input.generating:
        # return True
        mode = self.window.core.config.get('mode')
        if mode == "assistant":
            return True
        return False



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.26 10:00:00                  #
# ================================================== #

from pygpt_net.utils import trans
import pygpt_net.icons_rc

class Tools:
    def __init__(self, window=None):
        """
        Tools menu setup

        :param window: Window instance
        """
        self.window = window

    def setup(self):
        """Setup tools menu"""
        actions = self.window.tools.setup_menu_actions()
        if len(actions) == 0:
            return
        self.window.ui.menu['menu.tools'] = self.window.menuBar().addMenu(trans("menu.tools"))
        for key in actions:
            self.window.ui.menu[key] = actions[key]
            self.window.ui.menu['menu.tools'].addAction(self.window.ui.menu[key])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.08 21:00:00                  #
# ================================================== #

from PySide6.QtCore import QTimer, Signal, Slot, QThreadPool, QEvent, Qt
from PySide6.QtWidgets import QApplication, QMainWindow
from qt_material import QtStyleTools

from pygpt_net.container import Container
from pygpt_net.controller import Controller
from pygpt_net.tools import Tools
from pygpt_net.ui import UI
from pygpt_net.utils import get_app_meta


class MainWindow(QMainWindow, QtStyleTools):

    # states
    STATE_IDLE = 'idle'
    STATE_BUSY = 'busy'
    STATE_ERROR = 'error'

    # signals
    statusChanged = Signal(str)
    stateChanged = Signal(str)
    logger_message = Signal(object)

    def __init__(self, app: QApplication, args: dict = None):
        """
        Main window

        :param app: QApplication instance
        :param args: launcher arguments
        """
        super().__init__()
        self.app = app
        self.args = args
        self.timer = None
        self.post_timer = None
        self.update_timer = None
        self.threadpool = None
        self.is_closing = False
        self.timer_interval = 30
        self.post_timer_interval = 1000
        self.update_timer_interval = 300000  # check every 5 minutes
        self.state = self.STATE_IDLE
        self.prevState = None

        # load version info
        self.meta = get_app_meta()

        # setup service container
        self.core = Container(self)
        self.core.init()
        self.core.patch()  # patch version if needed

        # setup thread pool
        self.threadpool = QThreadPool()

        # setup controller
        self.controller = Controller(self)

        # setup tools
        self.tools = Tools(self)

        # init, load settings options, etc.
        self.controller.init()

        # setup UI
        self.ui = UI(self)
        self.ui.init()

        # setup signals
        self.statusChanged.connect(self.update_status)
        self.stateChanged.connect(self.update_state)

    def add_plugin(self, plugin):
        """
        Add plugin to app

        :param plugin: plugin instance
        """
        self.core.plugins.register(plugin)

    def add_llm(self, llm):
        """
        Add Langchain LLM wrapper to app

        :param llm: LLM wrapper instance
        """
        self.core.llm.register(llm.id, llm)

    def add_vector_store(self, store):
        """
        Add vector store provider to app

        :param store: Vector store provider instance
        """
        self.core.idx.storage.register(store.id, store)

    def add_loader(self, loader):
        """
        Add data loader to app

        :param loader: data loader instance
        """
        self.core.idx.indexing.register_loader(loader)

    def add_audio_input(self, provider):
        """
        Add audio input provider to app

        :param provider: audio input provider instance
        """
        self.core.audio.register(provider, "input")

    def add_audio_output(self, provider):
        """
        Add audio output provider to app

        :param provider: audio output provider instance
        """
        self.core.audio.register(provider, "output")

    def add_web(self, provider):
        """
        Add web provider to app

        :param provider: web provider instance
        """
        self.core.web.register(provider)

    def add_tool(self, tool):
        """
        Add tool to app

        :param tool: tool instance
        """
        self.tools.register(tool)

    def setup(self):
        """Setup app"""
        self.controller.setup()
        self.tools.setup()
        self.controller.plugins.setup()
        self.controller.post_setup()

    def post_setup(self):
        """Called after setup"""
        self.controller.layout.post_setup()
        self.timer = QTimer()
        self.timer.timeout.connect(self.update)
        self.timer.start(self.timer_interval)
        self.post_timer = QTimer()
        self.post_timer.timeout.connect(self.post_update)
        self.post_timer.start(self.post_timer_interval)
        self.update_timer = QTimer()
        self.update_timer.timeout.connect(self.core.updater.run_check)
        self.update_timer.start(self.update_timer_interval)
        self.logger_message.connect(self.controller.debug.handle_log)
        self.ui.post_setup()
        self.tools.post_setup()

    def update(self):
        """Called on every update"""
        self.controller.on_update()
        self.controller.plugins.on_update()
        self.tools.on_update()

    def post_update(self):
        """Called on post-update (slow)"""
        self.controller.debug.on_post_update()
        self.controller.plugins.on_post_update()
        self.tools.on_post_update()

    @Slot(str)
    def update_status(self, text: str):
        """
        Update status text

        :param text: status text
        """
        self.ui.status(text)

    @Slot(str)
    def update_state(self, state: str):
        """
        Update state

        :param state: state
        """
        self.state = state
        self.ui.tray.set_icon(state)

    def closeEvent(self, event):
        """
        Handle close event

        :param event: close event
        """
        if self.core.config.get('layout.tray.minimize') and self.ui.tray.is_tray:
            event.ignore()
            self.hide()
            return

        self.is_closing = True
        print("Closing...")
        print("Sending terminate signal to all plugins...")
        self.controller.chat.common.stop(exit=True)
        self.controller.plugins.destroy()
        print("Saving ctx groups...")
        self.controller.ctx.save_all()
        print("Saving notepad...")
        self.controller.notepad.save_all()
        print("Saving calendar...")
        self.controller.calendar.save_all()
        print("Saving drawing...")
        self.controller.painter.save_all()
        print("Saving tools...")
        self.tools.on_exit()
        print("Saving layout state...")
        self.controller.layout.save()
        print("Stopping timers...")
        self.timer.stop()
        self.post_timer.stop()
        self.update_timer.stop()
        print("Saving config...")
        self.core.config.save()
        print("Saving presets...")
        self.core.presets.save_all()
        print("Exiting...")
        event.accept()

    def changeEvent(self, event):
        """
        Handle window state change event

        :param event: Event
        """
        if event.type() == QEvent.WindowStateChange:
            if self.isMinimized() and self.core.config.get('layout.tray.minimize'):
                self.ui.tray_menu['restore'].setVisible(True)
                self.hide()
                event.ignore()
            else:
                self.prevState = event.oldState()

    def tray_toggle(self):
        """Toggle tray icon"""
        if self.core.config.get('layout.tray.minimize'):
            if self.isVisible():
                if not self.isActiveWindow():
                    self.activateWindow()
                else:
                    self.ui.tray_menu['restore'].setVisible(True)
                    self.hide()
            else:
                self.restore()
        else:
            if self.isMinimized():
                self.restore()
            else:
                if not self.isActiveWindow():
                    self.activateWindow()
                else:
                    self.showMinimized()

    def restore(self):
        """Restore window"""
        if self.prevState == Qt.WindowMaximized or self.windowState() == Qt.WindowMaximized:
            self.showMaximized()
        else:
            self.showNormal()
        self.activateWindow()
        self.ui.tray_menu['restore'].setVisible(False)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.09 23:00:00                  #
# ================================================== #

from PySide6 import QtCore
from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDialog, QPushButton, QHBoxLayout, QLabel, QVBoxLayout, QLineEdit

from pygpt_net.utils import trans
from .base import BaseDialog
from ..element.labels import HelpLabel
from ..option.input import DirectoryInput


class ProfileDialog(BaseDialog):
    def __init__(self, window=None, id=None):
        """
        Profile dialog

        :param window: main window
        :param id: settings id
        """
        super(ProfileDialog, self).__init__(window, id)
        self.window = window
        self.id = id

    def closeEvent(self, event):
        """
        Close event

        :param event: close event
        """
        self.window.controller.settings.profile.dialog = False
        self.window.controller.settings.profile.update_menu()
        super(ProfileDialog, self).closeEvent(event)

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key press event
        """
        if event.key() == Qt.Key_Escape:
            self.cleanup()
            self.close()  # close dialog when the Esc key is pressed.
        else:
            super(ProfileDialog, self).keyPressEvent(event)

    def cleanup(self):
        """
        Cleanup on close
        """
        self.window.controller.settings.profile.dialog = False
        self.window.controller.settings.profile.update_menu()



class ProfileEditDialog(QDialog):
    def __init__(self, window=None, id=None):
        """
        Profile item edit dialog

        :param window: main window
        """
        super(ProfileEditDialog, self).__init__(window)
        self.window = window
        self.id = id
        self.uuid = None
        self.path = None
        self.name = ""
        self.mode = "create"  # create | update
        self.input = ProfileNameInput(window, id)
        self.input.setMinimumWidth(400)

        self.window.ui.nodes['dialog.profile.item.btn.update'] = QPushButton(trans('dialog.profile.item.btn.update'))
        self.window.ui.nodes['dialog.profile.item.btn.update'].clicked.connect(
            lambda: self.window.controller.settings.profile.handle_update(
                self.mode,
                self.input.text(),
                self.workdir.text(),
                self.uuid,
            )
        )

        self.window.ui.nodes['dialog.profile.item.btn.dismiss'] = QPushButton(trans('dialog.profile.item.btn.dismiss'))
        self.window.ui.nodes['dialog.profile.item.btn.dismiss'].clicked.connect(
            lambda: self.window.controller.settings.profile.dismiss_update()
        )

        bottom = QHBoxLayout()
        bottom.addWidget(self.window.ui.nodes['dialog.profile.item.btn.dismiss'])
        bottom.addWidget(self.window.ui.nodes['dialog.profile.item.btn.update'])

        self.window.ui.nodes['dialog.profile.name.label'] = QLabel(trans("dialog.profile.name.label"))
        self.window.ui.nodes['dialog.profile.workdir.label'] = QLabel(trans("dialog.profile.workdir.label"))

        option = {
            'type': 'text',
            'label': 'Directory',
            'value': "",
        }
        self.workdir = DirectoryInput(self.window, 'profile', 'item', option)

        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.nodes['dialog.profile.name.label'])
        layout.addWidget(self.input)
        layout.addWidget(self.window.ui.nodes['dialog.profile.workdir.label'])
        layout.addWidget(self.workdir)
        layout.addLayout(bottom)

        self.setLayout(layout)

    def prepare(self):
        """Prepare dialog before show"""
        if self.mode == 'create':
            self.window.ui.nodes['dialog.profile.item.btn.update'].setText(trans('dialog.profile.item.btn.create'))
        elif self.mode == 'edit':
            self.window.ui.nodes['dialog.profile.item.btn.update'].setText(trans("dialog.profile.item.btn.update"))
        elif self.mode == 'duplicate':
            self.window.ui.nodes['dialog.profile.item.btn.update'].setText(trans("dialog.profile.item.btn.duplicate"))

        self.workdir.setText(self.path)


class ProfileNameInput(QLineEdit):
    def __init__(self, window=None, id=None):
        """
        Profile name dialog input

        :param window: main window
        :param id: info window id
        """
        super(ProfileNameInput, self).__init__(window)

        self.window = window
        self.id = id

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key event
        """
        super(ProfileNameInput, self).keyPressEvent(event)

        # save on Enter
        if event.key() == QtCore.Qt.Key_Return or event.key() == QtCore.Qt.Key_Enter:
            self.window.controller.settings.profile.handle_update(
                self.window.ui.dialog['profile.item'].mode,
                self.window.ui.dialog['profile.item'].input.text(),
                self.window.ui.dialog['profile.item'].workdir.text(),
                self.window.ui.dialog['profile.item'].uuid,
            )



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.16 12:00:00                  #
# ================================================== #

from pygpt_net.item.ctx import CtxItem
from pygpt_net.plugin.base import BasePlugin
from pygpt_net.core.dispatcher import Event


class Plugin(BasePlugin):
    def __init__(self, *args, **kwargs):
        super(Plugin, self).__init__(*args, **kwargs)
        self.id = "openai_vision"
        self.name = "GPT-4 Vision (inline)"
        self.type = [
            "vision",
        ]
        self.description = "Integrates GPT-4 Vision abilities with any chat mode"
        self.order = 100
        self.use_locale = True
        self.prompt = ""
        self.allowed_urls_ext = [
            ".jpg",
            ".png",
            ".jpeg",
            ".gif",
            ".webp",
        ]
        self.allowed_cmds = [
            "camera_capture",
            "make_screenshot",
        ]
        self.init_options()

    def init_options(self):
        prompt = "IMAGE ANALYSIS: You are an expert in analyzing photos. Each time I send you a photo, you will " \
                 "analyze it as accurately as you can, answer all my questions, and offer any help on the subject " \
                 "related to the photo.  Remember to always describe in great detail all the aspects related to the " \
                 "photo, try to place them in the context of the conversation, and make a full analysis of what you " \
                 "see. "
        self.add_option(
            "model",
            type="combo",
            use="models",
            value="gpt-4-vision-preview",
            label="Model",
            description="Model used to temporarily providing vision abilities, "
                        "default: gpt-4-vision-preview",
            tooltip="Model",
        )
        self.add_option(
            "prompt",
            type="textarea",
            value=prompt,
            label="Prompt",
            description="Prompt used for vision mode. It will append or replace current system prompt "
                        "when using vision model",
            tooltip="Prompt",
            advanced=True,
        )
        self.add_option(
            "replace_prompt",
            type="bool",
            value=False,
            label="Replace prompt",
            description="Replace whole system prompt with vision prompt against appending "
                        "it to the current prompt",
            tooltip="Replace whole system prompt with vision prompt against appending it to the "
                    "current prompt",
            advanced=True,
        )
        self.add_cmd(
            "camera_capture",
            instruction="capture image from webcam",
            params=[],
            enabled=True,
        )
        self.add_cmd(
            "make_screenshot",
            instruction="make desktop screenshot",
            params=[],
            enabled=True,
        )

    def setup(self) -> dict:
        """
        Return available config options

        :return: config options
        """
        return self.options

    def attach(self, window):
        """
        Attach window

        :param window: Window instance
        """
        self.window = window

    def is_allowed(self, mode: str) -> bool:
        """
        Check if plugin is allowed in given mode

        :param mode: mode name
        :return: True if allowed, False otherwise
        """
        return mode in self.window.controller.chat.vision.allowed_modes

    def handle(self, event: Event, *args, **kwargs):
        """
        Handle dispatched event

        :param event: event object
        :param args: event args
        :param kwargs: event kwargs
        """
        name = event.name
        data = event.data
        ctx = event.ctx

        if name == Event.MODE_BEFORE:
            if self.is_allowed(data['value']):
                data['value'] = self.on_mode_before(
                    ctx,
                    mode=data['value'],
                )  # mode change

        elif name == Event.MODEL_BEFORE:
            if "mode" in data and data["mode"] == "vision":
                key = self.get_option_value("model")
                if self.window.core.models.has(key):
                    data['model'] = self.window.core.models.get(key)

        elif name == Event.PRE_PROMPT:
            if self.is_allowed(data['mode']):
                data['value'] = self.on_pre_prompt(data['value'])

        elif name == Event.INPUT_BEFORE:
            self.prompt = str(data['value'])

        elif name == Event.SYSTEM_PROMPT:
            if self.is_allowed(data['mode']):
                data['value'] = self.on_system_prompt(data['value'])

        elif name == Event.UI_ATTACHMENTS:
            data['value'] = True  # allow render attachments UI elements

        elif name == Event.UI_VISION:
            if self.is_allowed(data['mode']):
                data['value'] = True  # allow render vision UI elements

        elif name in [
            Event.CTX_SELECT,
            Event.MODE_SELECT,
            Event.MODEL_SELECT,
        ]:
            self.on_toggle(False)  # always reset vision flag / disable vision mode

        elif name == Event.CMD_SYNTAX:
            self.cmd_syntax(data)

        elif name == Event.CMD_EXECUTE:
            self.cmd(
                ctx,
                data['commands'],
            )

    def cmd_syntax(self, data: dict):
        """
        Event: CMD_SYNTAX

        :param data: event data dict
        """
        if not self.has_cmd("camera_capture") and not self.has_cmd("make_screenshot"):
            return
        if self.has_cmd("camera_capture"):
            data['cmd'].append(self.get_cmd("camera_capture"))  # append command
        if self.has_cmd("make_screenshot"):
            data['cmd'].append(self.get_cmd("make_screenshot"))  # append command

    def cmd(self, ctx: CtxItem, cmds: list):
        """
        Event: CMD_EXECUTE

        :param ctx: CtxItem
        :param cmds: commands dict
        """
        is_cmd = False
        my_commands = []
        for item in cmds:
            if item["cmd"] in self.allowed_cmds:
                my_commands.append(item)
                is_cmd = True

        if not is_cmd:
            return

        responses = []
        for item in my_commands:
            response = None

            if item["cmd"] == "camera_capture" and self.has_cmd(item["cmd"]):
                response = self.cmd_camera_capture(item)

            elif item["cmd"] == "make_screenshot" and self.has_cmd(item["cmd"]):
                response = self.cmd_make_screenshot(item)

            if response:
                responses.append(response)

        # send response
        if len(responses) > 0:
            for response in responses:
                self.reply(response, ctx)

    def cmd_camera_capture(self, item: dict) -> dict:
        """
        Capture image from camera

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        self.window.controller.camera.manual_capture(force=True)
        response = {
            "request": request,
            "result": "OK",
        }
        return response

    def cmd_make_screenshot(self, item: dict) -> dict:
        """
        Make desktop screenshot

        :param item: command item
        :return: response item
        """
        request = self.prepare_request(item)
        self.window.controller.painter.capture.screenshot()
        response = {
            "request": request,
            "result": "OK",
        }
        return response

    def prepare_request(self, item) -> dict:
        """
        Prepare request item for result

        :param item: item with parameters
        :return: request item
        """
        return {"cmd": item["cmd"]}

    def on_toggle(self, value: bool):
        """
        Events: CTX_SELECT, MODE_SELECT, MODEL_SELECT

        :param value: vision mode state
        """
        if not value:
            self.window.controller.chat.vision.is_enabled = False

    def on_system_prompt(self, prompt: str) -> str:
        """
        Event: SYSTEM_PROMPT

        :param prompt: prompt
        :return: updated prompt
        """
        # append vision prompt only if vision is provided or enabled
        if not self.is_vision_provided() \
                and not self.window.controller.chat.vision.enabled():
            return prompt

        # append vision prompt
        if not self.get_option_value("replace_prompt"):
            prompt += "\n" + self.get_option_value("prompt")
        return prompt

    def on_pre_prompt(self, prompt: str) -> str:
        """
        Event: PRE_PROMPT

        :param prompt: prompt
        :return: updated prompt
        """
        # append vision prompt only if vision is provided or enabled
        if not self.is_vision_provided() \
                and not self.window.controller.chat.vision.enabled():
            return prompt

        # replace vision prompt
        if self.get_option_value("replace_prompt"):
            return self.get_option_value("prompt")
        else:
            return prompt

    def is_vision_provided(self) -> bool:
        """
        Check if content for vision is provided (images, attachments)

        :return: True if vision is provided in this ctx
        """
        result = False
        mode = self.window.core.config.get('mode')
        attachments = self.window.core.attachments.get_all(mode)  # from global mode
        self.window.core.gpt.vision.build_content(
            str(self.prompt),
            attachments,
        )  # tmp build content, provide attachments from global mode

        built_attachments = self.window.core.gpt.vision.attachments
        built_urls = self.window.core.gpt.vision.urls

        # check for images in URLs found in prompt
        img_urls = []
        for url in built_urls:
            for ext in self.allowed_urls_ext:
                if url.lower().endswith(ext):
                    img_urls.append(url)
                    break

        if len(built_attachments) > 0 or len(img_urls) > 0:
            result = True

        return result

    def on_mode_before(self, ctx: CtxItem, mode: str) -> str:
        """
        Event: MODE_BEFORE

        :param ctx: current ctx
        :param mode: current mode
        :return: updated mode
        """
        # abort if already in vision mode
        if mode == 'vision':
            return mode  # keep current mode

        # if already used in this ctx then keep vision mode
        if self.window.controller.chat.vision.enabled():
            ctx.is_vision = True
            return 'vision'

        if self.is_vision_provided():
            self.window.controller.chat.vision.enable()
            ctx.is_vision = True
            return 'vision'  # jump to vision mode (only for this call)

        return mode  # keep current mode

    def log(self, msg: str):
        """
        Log message to console

        :param msg: message to log
        """
        full_msg = '[Vision] ' + str(msg)
        self.debug(full_msg)
        self.window.ui.status(full_msg)
        if self.is_log():
            print(full_msg)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.12 21:00:00                  #
# ================================================== #

class Placeholder:
    def __init__(self, window=None):
        """
        Configuration placeholder options controller

        :param window: Window instance
        """
        self.window = window

    def apply(self, option: dict):
        """
        Apply placeholders to option

        :param option: Option dict
        """
        if option['type'] == 'dict' and 'keys' in option:
            for key in option['keys']:
                item = option['keys'][key]
                if type(item) is dict:
                    if "type" in item:
                        if item["type"] == "combo":
                            if "use" in item and item["use"] is not None:
                                item["keys"] = self.apply_by_id(item["use"])
        elif option['type'] == 'cmd' and 'params_keys' in option:
            for key in option['params_keys']:
                item = option['params_keys'][key]
                if type(item) is dict:
                    if "type" in item:
                        if item["type"] == "combo":
                            if "use" in item and item["use"] is not None:
                                item["keys"] = self.apply_by_id(item["use"])
        elif option['type'] == 'combo':
            if "use" in option and option["use"] is not None:
                option["keys"] = self.apply_by_id(option["use"])

    def apply_by_id(self, id: str) -> list:
        """
        Apply placeholders by id

        :param id: Placeholder options id
        """
        if id == "presets":
            return self.get_presets()
        elif id == "models":
            return self.get_models()
        elif id == "langchain_providers":
            return self.get_langchain_providers()
        elif id == "llama_index_providers":
            return self.get_llama_index_providers()
        elif id == "embeddings_providers":
            return self.get_embeddings_providers()
        elif id == "llama_index_loaders":
            return self.get_llama_index_loaders()
        elif id == "llama_index_loaders_file":
            return self.get_llama_index_loaders(type="file")
        elif id == "llama_index_loaders_web":
            return self.get_llama_index_loaders(type="web")
        elif id == "vector_storage":
            return self.get_vector_storage()
        elif id == "var_types":
            return self.get_var_types()
        elif id == "agent_modes":
            return self.get_agent_modes()
        elif id == "idx":
            return self.get_idx()
        else:
            return []

    def get_langchain_providers(self) -> list:
        """
        Get langchain placeholders list

        :return: placeholders list
        """
        ids = self.window.core.llm.get_ids("langchain")
        data = []
        data.append({'_': '---'})
        for id in ids:
            data.append({id: id})
        return data

    def get_llama_index_providers(self) -> list:
        """
        Get llama placeholders list

        :return: placeholders list
        """
        ids = self.window.core.llm.get_ids("llama_index")
        data = []
        data.append({'_': '---'})
        for id in ids:
            data.append({id: id})
        return data

    def get_embeddings_providers(self) -> list:
        """
        Get embeddings placeholders list

        :return: placeholders list
        """
        ids = self.window.core.llm.get_ids("embeddings")
        data = []
        data.append({'_': '---'})
        for id in ids:
            data.append({id: id})
        return data

    def get_llama_index_loaders(self, type: str = "all") -> list:
        """
        Get data loaders placeholders list

        :param type: data type
        :return: placeholders list
        """
        data = []
        choices = self.window.controller.idx.common.get_loaders_choices() # list
        for choice in choices:
            if type == "all":
                data.append(choice)
            elif type == "file":
                key = list(choice.keys())[0]
                if key.startswith("file_"):
                    data.append(choice)
            elif type == "web":
                key = list(choice.keys())[0]
                if key.startswith("web_"):
                    data.append(choice)
        return data

    def get_vector_storage(self) -> list:
        """
        Get vector storage placeholders list

        :return: placeholders list
        """
        ids = self.window.core.idx.storage.get_ids()
        data = []
        for id in ids:
            data.append({id: id})
        return data

    def get_var_types(self) -> list:
        """
        Get langchain placeholders list

        :return: placeholders list
        """
        types = ["str", "int", "float", "bool", "dict", "list", "None"]
        data = []
        for type in types:
            data.append({type: type})
        return data

    def get_presets(self) -> list:
        """
        Get presets placeholders list

        :return: Presets placeholders list
        """
        presets = self.window.core.presets.get_all()
        data = []
        data.append({'_': '---'})
        for id in presets:
            if id.startswith("current."):  # ignore "current" preset
                continue
            data.append({id: id})  # TODO: name
        return data

    def get_models(self) -> list:
        """
        Get models placeholders list

        :return: Models placeholders list
        """
        models = self.window.core.models.get_all()
        data = []
        data.append({'_': '---'})
        for id in models:
            data.append({id: id})  # TODO: name
        return data

    def get_agent_modes(self) -> list:
        """
        Get modes placeholders list

        :return: Models placeholders list
        """
        modes = ["chat", "completion", "vision", "langchain", "llama_index"]
        data = []
        for id in modes:
            data.append({id: id})  # TODO: name
        return data

    def get_idx(self) -> list:
        """
        Get indexes placeholders list

        :return: Indexes placeholders list
        """
        indexes = self.window.core.idx.get_idx_ids()
        data = []
        data.append({'_': '---'})
        for id in indexes:
            data.append({id: id})
        return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.27 21:00:00                  #
# ================================================== #

from sqlalchemy import text

from .base import BaseMigration


class Version20231227152900(BaseMigration):
    def __init__(self, window=None):
        super(Version20231227152900, self).__init__(window)
        self.window = window

    def up(self, conn):
        conn.execute(text("""
        CREATE TABLE IF NOT EXISTS ctx_meta (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            external_id TEXT,
            uuid TEXT,
            created_ts INTEGER,
            updated_ts INTEGER,
            name TEXT,
            mode TEXT,
            model TEXT,
            last_mode TEXT,
            last_model TEXT,
            thread_id TEXT,
            assistant_id TEXT,
            preset_id TEXT,
            run_id TEXT,
            status TEXT,
            extra TEXT,
            is_initialized BOOLEAN NOT NULL CHECK (is_initialized IN (0, 1)),
            is_deleted BOOLEAN NOT NULL CHECK (is_deleted IN (0, 1)),
            is_important BOOLEAN NOT NULL CHECK (is_important IN (0, 1)),
            is_archived BOOLEAN NOT NULL CHECK (is_archived IN (0, 1))
        );"""))

        conn.execute(text("""
        CREATE TABLE IF NOT EXISTS ctx_item (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            external_id TEXT,
            meta_id INTEGER,
            input TEXT,
            output TEXT,
            input_name TEXT,
            output_name TEXT,
            input_ts INTEGER,
            output_ts INTEGER,
            mode TEXT,
            model TEXT,
            thread_id TEXT,
            msg_id TEXT,
            run_id TEXT,
            cmds_json TEXT,
            results_json TEXT,
            urls_json TEXT,
            images_json TEXT,
            files_json TEXT,
            attachments_json TEXT,
            extra TEXT,
            input_tokens INTEGER,
            output_tokens INTEGER,
            total_tokens INTEGER,
            FOREIGN KEY(meta_id) REFERENCES ctx_meta(id) ON DELETE SET NULL
        );
        """))

        conn.execute(text("""
        CREATE TABLE IF NOT EXISTS notepad (
            id INTEGER PRIMARY KEY AUTOINCREMENT,
            idx INTEGER,
            uuid TEXT,
            created_ts INTEGER,
            updated_ts INTEGER,
            title TEXT,
            content TEXT,
            is_deleted BOOLEAN NOT NULL CHECK (is_deleted IN (0, 1))
        );"""))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.26 15:00:00                  #
# ================================================== #

import os

from PySide6.QtCore import Qt, QUrl, QTimer
from PySide6.QtGui import QIcon, QAction
from PySide6.QtMultimedia import QMediaPlayer, QAudioOutput
from PySide6.QtMultimediaWidgets import QVideoWidget
from PySide6.QtWidgets import (QWidget, QPushButton, QHBoxLayout, QVBoxLayout,
                               QFileDialog, QSlider, QLabel, QStyle, QSizePolicy, QMenu, QMessageBox)

from pygpt_net.ui.widget.element.labels import HelpLabel
from pygpt_net.utils import trans

import pygpt_net.icons_rc

class VideoPlayerWidget(QWidget):
    def __init__(self, window=None):
        super().__init__(window)
        self.window = window
        self.player = QMediaPlayer(window)
        self.loaded = False
        self.path = None
        self.stopped = False
        self.seeking = False

        self.update_timer = QTimer(self)
        self.update_timer.setInterval(200)
        self.update_timer.timeout.connect(self.update_ui)
        self.autoplay_timer = QTimer(self)
        self.autoplay_timer.setInterval(1000)
        self.autoplay_timer.setSingleShot(True)
        self.autoplay_timer.timeout.connect(self.after_loaded)

        self.audio = QAudioOutput()
        self.video = QVideoWidget(window)
        self.video.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.video.setMinimumSize(640, 480)

        self.btn_play_pause = QPushButton()
        self.btn_play_pause.setEnabled(False)

        self.btn_stop = QPushButton()
        self.btn_stop.setIcon(self.style().standardIcon(QStyle.SP_MediaStop))
        self.btn_stop.setEnabled(False)

        self.btn_mute = QPushButton()
        self.btn_mute.setIcon(QIcon(':/icons/mute.svg'))
        self.btn_mute.setCheckable(True)

        self.btn_open = QPushButton(trans("action.video.open"))

        self.label_duration = QLabel()
        self.label_time = QLabel()
        self.label_path = HelpLabel("")
        self.label_path.setWordWrap(False)

        self.slider = QSlider(Qt.Horizontal)
        self.slider.setRange(0, 0)

        self.volume_slider = QSlider(Qt.Horizontal)
        self.volume_slider.setRange(0, 100)
        self.volume_slider.setValue(100)
        self.volume_slider.setToolTip('Adjust volume')

        layout_volume = QHBoxLayout()
        layout_volume.addWidget(self.volume_slider)
        layout_volume.addWidget(self.btn_mute)

        layout_bottom = QHBoxLayout()
        layout_bottom.addWidget(self.label_path)
        layout_bottom.addStretch()
        layout_bottom.addLayout(layout_volume)

        # layout
        layout_controls = QHBoxLayout()
        layout_controls.addWidget(self.btn_open)
        layout_controls.addWidget(self.btn_play_pause)
        layout_controls.addWidget(self.btn_stop)

        layout_slider = QHBoxLayout()
        layout_slider.addWidget(self.label_time)
        layout_slider.addWidget(self.slider)
        layout_slider.addWidget(self.label_duration)

        layout = QVBoxLayout()
        layout.addWidget(self.video)
        layout.addLayout(layout_slider)
        layout.addLayout(layout_controls)
        layout.addLayout(layout_bottom)

        self.setLayout(layout)
        self.setContextMenuPolicy(Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.on_context_menu)

        self.update_play_pause_icon()
        self.bind_signals()

    def bind_signals(self):
        """Bind signals"""
        self.btn_open.clicked.connect(self.open_file)
        self.btn_play_pause.clicked.connect(self.toggle_play_pause)
        self.btn_stop.clicked.connect(self.stop_video)
        self.btn_mute.toggled.connect(self.set_muted)
        self.player.positionChanged.connect(self.position_changed)
        self.player.durationChanged.connect(self.duration_changed)
        self.player.playbackStateChanged.connect(self.on_playback_state_changed)
        self.player.mediaStatusChanged.connect(self.media_status_changed)
        self.video.mousePressEvent = self.video_widget_clicked

        self.player.setAudioOutput(self.audio)
        self.player.setVideoOutput(self.video)
        self.volume_slider.valueChanged.connect(self.adjust_volume)
        self.slider.sliderPressed.connect(self.on_slider_pressed)
        self.slider.sliderReleased.connect(self.on_slider_released)
        self.seeking = False

    def update(self):
        """Update player"""
        self.update_mute_icon()
        self.update_volume_slider()
        self.update_label_path()

    def open_file(self):
        """Open file"""
        path, _ = QFileDialog.getOpenFileName(
            self,
            trans("action.video.open"),
            "",
            "Video and Audio Files (*.mp4 *.avi *.mkv *.mp3 *.wav *.flac *.ogg *.m4a *.aac *.wma *.webm *.mkv "
            "*.flv *.mov *.wmv *.3gp *.m4v *.mpg);;All Files (*)"
        )
        if path:
            self.open(path)

    def open(self, path: str):
        """
        Open video

        :param path: video path
        """
        self.loaded = False
        self.stopped = False
        self.seeking = False
        self.reset()
        self.player.setSource(QUrl.fromLocalFile(path))
        self.set_path(path)
        self.enable()
        self.autoplay_timer.start()
        self.update()
        self.window.tools.get("player").store_path(path)  # save

    def enable(self):
        """Enable player"""
        self.btn_play_pause.setEnabled(True)
        self.btn_stop.setEnabled(True)

    def play_video(self):
        """Play video"""
        self.stopped = False
        if not self.loaded and self.path:
            self.open(self.path)
        else:
            self.update_audio()
            self.player.play()

    def stop_video(self):
        """Stop video"""
        if self.player.source():
            self.player.stop()
        self.stopped = True
        self.slider.setValue(0)
        self.label_time.setText(self.format_time(0))
        self.btn_play_pause.setIcon(self.style().standardIcon(QStyle.SP_MediaPlay))
        self.update_timer.stop()

    def toggle_play_pause(self):
        """Toggle play/pause"""
        if self.player.playbackState() == QMediaPlayer.PlayingState:
            self.player.pause()
        else:
            self.play_video()

        self.on_playback_state_changed()

    def reset(self):
        """Reset player to default state"""
        if self.player.source():
            self.player.stop()
            self.player.setSource(QUrl())
        self.btn_play_pause.setIcon(self.style().standardIcon(QStyle.SP_MediaPlay))
        self.update_timer.stop()
        self.slider.setValue(0)
        self.seeking = False
        self.loaded = False

    def reset_player(self):
        """Reset player to default state"""
        if self.player is not None:
            if self.player.playbackState() == QMediaPlayer.PlayingState:
                self.player.stop()
            self.player.setSource(QUrl())

    def on_context_menu(self, point):
        """
        Context menu event

        :param point: point
        """
        if not self.path or not self.player.source():
            return

        # TODO: implement grab screenshot
        """
        use_actions = []
        action_as_attachment = QAction(
            QIcon(":/icons/attachment.svg"),
            trans('action.use.attachment'),
            self,
        )
        action_as_attachment.triggered.connect(
            lambda: self.use_as_attachment(),
        )
        action_as_image = QAction(
            QIcon(":/icons/image.svg"),
            trans('action.use.image'),
            self,
        )
        action_as_image.triggered.connect(
            lambda: self.use_as_image(),
        )
        use_actions.append(action_as_image)
        use_actions.append(action_as_attachment)

        # use by type
        if use_actions:
            # use menu
            use_menu = QMenu(trans('action.use'), self)
            for action in use_actions:
                use_menu.addAction(action)
            context_menu.addMenu(use_menu)
        """
        context_menu = QMenu(self)
        save_as_action = QAction(QIcon(":/icons/save.svg"), trans("action.save_as"), self)
        save_as_action.triggered.connect(
            lambda: self.window.tools.get("player").save_as_file()
        )
        # full_screen_action = QAction('Fullscreen', self)
        # full_screen_action.triggered.connect(self.toggle_fullscreen)
        context_menu.addAction(save_as_action)

        # context_menu.addAction(full_screen_action)
        context_menu.exec(self.mapToGlobal(point))

    def use_as_attachment(self):
        """Use as attachment"""
        # TODO: implement grab screenshot
        path = self.window.tools.get("player").grab_frame()
        self.window.controller.files.use_attachment(path)

    def use_as_image(self):
        """Use as image"""
        # TODO: implement grab screenshot
        path = self.window.tools.get("player").grab_frame()
        self.window.controller.painter.open_external(path)

    def adjust_volume(self, value):
        """
        Adjust volume

        :param value: volume value
        """
        self.window.core.config.set("video.player.volume", value)  # save
        volume = value / 100.0
        self.audio.setVolume(volume)

    def set_path(self, path: str):
        """
        Set path to video

        :param path: path
        """
        self.path = path
        self.update_label_path()
        if not self.loaded and self.path:
            if not self.player.source():
                self.reset()
                self.player.setSource(QUrl.fromLocalFile(self.path))
            self.enable()
            self.update()
        self.window.ui.dialog["video_player"].setWindowTitle(os.path.basename(self.path))

    def on_close(self):
        """Stop video"""
        self.reset()

    def toggle_fullscreen(self):
        """Toggle fullscreen"""
        # TODO: implement fullscreen
        if self.window.ui.dialog['video_player'].isFullScreen():
            self.window.ui.dialog['video_player'].showNormal()
        else:
            self.window.ui.dialog['video_player'].showFullScreen()

    def media_status_changed(self, status):
        """
        Media status changed

        :param status: status
        """
        if status == QMediaPlayer.LoadedMedia:
            self.loaded = True
            self.force_resize()
        elif status == QMediaPlayer.InvalidMedia:
            self.loaded = False
            self.window.ui.dialogs.alert("Failed to load media file, missing video codec?")

    def after_loaded(self):
        """Start playback after loaded"""
        if self.player.mediaStatus() == QMediaPlayer.LoadedMedia:
            self.loaded = True
            if self.player.playbackState() != QMediaPlayer.PlayingState:
                self.toggle_play_pause()
                self.force_resize()

    def force_resize(self):
        """Force resize fix"""
        # fix for video not showing after re-opening the dialog
        self.video.hide()
        self.video.show()
        self.video.update()
        current_size = self.window.ui.dialog['video_player'].size()
        self.window.ui.dialog['video_player'].resize(current_size.width() + 10, current_size.height() + 10)
        self.window.ui.dialog['video_player'].resize(current_size)
        self.window.ui.dialog['video_player'].repaint()
        w = self.video.width() - 1
        h = self.video.height() - 1
        self.video.resize(w, h)

    def on_playback_state_changed(self):
        """Playback state changed"""
        self.update_play_pause_icon()

    def video_widget_clicked(self, event):
        """
        Video widget clicked

        :param event: event
        """
        if event.button() == Qt.LeftButton:
            if self.loaded:
                self.toggle_play_pause()
        elif event.button() == Qt.RightButton:
            pass

    def duration_changed(self, duration):
        """
        Duration changed

        :param duration: duration
        """
        self.slider.setRange(0, duration)
        self.label_duration.setText(self.format_time(duration))

    def set_muted(self, state: bool):
        """
        Set volume muted

        :param state: muted
        """
        self.audio.setMuted(state)
        self.window.core.config.set("video.player.volume.mute", state)  # save

    def set_position(self, position):
        """
        Set position

        :param position: position
        """
        if self.player.playbackState() == QMediaPlayer.PlayingState:
            self.player.pause()

        self.player.positionChanged.connect(self.on_position_set)
        self.player.setPosition(position)

    def on_slider_pressed(self):
        """Slider pressed"""
        self.seeking = True
        self.player.pause()

    def on_slider_released(self):
        """Slider released"""
        if self.seeking:
            self.player.setPosition(self.slider.value())
            self.update_audio()
            self.player.play()
            self.seeking = False

    def position_changed(self, position):
        """
        Position changed

        :param position: position
        """
        if not self.seeking:
            self.slider.setValue(position)
            self.label_time.setText(self.format_time(position))

    def on_position_set(self, position):
        """
        On position set

        :param position: position
        """
        if self.seeking:
            self.player.positionChanged.disconnect(self.on_position_set)
            if self.player.playbackState() == QMediaPlayer.PausedState:
                QTimer.singleShot(100, self.player.play)
            self.seeking = False

    def update_label_path(self):
        """Update label path"""
        if self.path:
            self.label_path.setText(os.path.basename(self.path))
        else:
            self.label_path.setText("")

    def update_play_pause_icon(self):
        """Update play/pause icon"""
        if self.player.playbackState() == QMediaPlayer.PlayingState:
            self.btn_play_pause.setIcon(self.style().standardIcon(QStyle.SP_MediaPause))
            self.update_timer.start()
        else:
            self.btn_play_pause.setIcon(self.style().standardIcon(QStyle.SP_MediaPlay))
            self.update_timer.stop()

    def update_audio(self):
        """Re-assign audio"""
        self.player.setAudioOutput(None)  # audio respawn fix
        self.player.setAudioOutput(self.audio)

    def update_mute_icon(self):
        """Update mute icon"""
        self.btn_mute.setChecked(self.audio.isMuted())

    def update_volume_slider(self):
        """Update volume slider"""
        self.volume_slider.setValue(self.audio.volume() * 100)

    def update_ui(self):
        """Update UI"""
        # sync slider with the video time
        self.position_changed(self.player.position())

    def format_time(self, ms):
        """
        Format time

        :param ms: milliseconds
        """
        # convert ms to H:M:S format
        seconds = (ms // 1000) % 60
        minutes = (ms // 60000) % 60
        hours = (ms // 3600000)
        return "{:02d}:{:02d}:{:02d}".format(hours, minutes, seconds)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.25 12:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QPushButton, QHBoxLayout, QLabel, QVBoxLayout, QSizePolicy

from pygpt_net.ui.widget.dialog.workdir import WorkdirDialog
from pygpt_net.ui.widget.option.input import DirectoryInput
from pygpt_net.utils import trans


class Workdir:
    def __init__(self, window=None):
        """
        Workdir change dialog

        :param window: Window instance
        """
        self.window = window
        self.path = None

    def setup(self):
        """Setup change workdir dialog"""
        self.window.ui.nodes['workdir.change.info'] = QLabel("")
        self.window.ui.nodes['workdir.change.info'].setWordWrap(True)
        self.window.ui.nodes['workdir.change.info'].setSizePolicy(
            QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding))
        self.window.ui.nodes['workdir.change.info'].setMinimumWidth(300)

        self.window.ui.nodes['workdir.change.update.btn'] = QPushButton(trans("dialog.workdir.update.btn"))
        self.window.ui.nodes['workdir.change.update.btn'].clicked.connect(
            lambda: self.change_directory())

        self.window.ui.nodes['workdir.change.reset.btn'] = QPushButton(trans("dialog.workdir.reset.btn"))
        self.window.ui.nodes['workdir.change.reset.btn'].clicked.connect(
            lambda: self.reset_directory())

        self.window.ui.nodes['workdir.change.status'] = QLabel("")
        self.window.ui.nodes['workdir.change.status'].setVisible(False)

        bottom_layout = QHBoxLayout()
        bottom_layout.addWidget(self.window.ui.nodes['workdir.change.reset.btn'])
        bottom_layout.addWidget(self.window.ui.nodes['workdir.change.update.btn'])

        option = {
            'type': 'text',
            'label': 'Directory',
            'value': self.window.core.config.get_user_path(),
        }
        self.path = DirectoryInput(self.window, 'config', 'workdir', option)

        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.nodes['workdir.change.info'])
        layout.addWidget(self.path)
        layout.addLayout(bottom_layout)
        layout.addWidget(self.window.ui.nodes['workdir.change.status'], alignment=Qt.AlignCenter)

        self.prepare()
        self.window.ui.dialog['workdir.change'] = WorkdirDialog(self.window)
        self.window.ui.dialog['workdir.change'].setLayout(layout)
        self.window.ui.dialog['workdir.change'].setWindowTitle(trans("dialog.workdir.title"))

    def prepare(self):
        """Prepare workdir change dialog"""
        size_needed = self.window.core.filesystem.get_directory_size(self.window.core.config.get_user_path())
        self.window.ui.nodes['workdir.change.info'].setText(trans("dialog.workdir.tip").format(size=size_needed))

    def show_status(self, status: str):
        """
        Show status message

        :param status: status message
        """
        self.window.ui.nodes['workdir.change.status'].setText(status)
        self.window.ui.nodes['workdir.change.status'].setVisible(True)

    def hide_status(self):
        """Hide status message"""
        self.window.ui.nodes['workdir.change.status'].setText("")
        self.window.ui.nodes['workdir.change.status'].setVisible(False)

    def change_directory(self):
        """Update working directory"""
        self.window.controller.settings.workdir.migrate(self.path.text())

    def reset_directory(self):
        """Reset working directory"""
        current = self.window.core.config.get_user_path()
        self.path.value = current
        self.path.setText(current)

    def set_path(self, path: str):
        """
        Set working directory path

        :param path: path to directory
        """
        self.path.setText(path)
        self.path.value = path
        self.path.update()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.25 12:00:00                  #
# ================================================== #

import os

from PySide6.QtGui import QTextCursor, Qt
from PySide6.QtWidgets import QPushButton, QHBoxLayout, QLabel, QVBoxLayout

from pygpt_net.ui.widget.dialog.applog import AppLogDialog
from pygpt_net.ui.widget.element.labels import TitleLabel
from pygpt_net.ui.widget.textarea.editor import CodeEditor


class AppLog:
    def __init__(self, window=None):
        """
        App log dialog

        :param window: Window instance
        """
        self.window = window

    def setup(self):
        """Setup app log dialog"""
        id = 'app.log'

        self.window.ui.editor[id] = CodeEditor(self.window)
        self.window.ui.editor[id].setReadOnly(True)
        self.window.ui.editor[id].setProperty('class', 'text-editor')

        self.window.ui.nodes['editor.app.log.btn.open'] = QPushButton("OPEN (EXTERNAL)")
        self.window.ui.nodes['editor.app.log.btn.clear'] = QPushButton("CLEAR")
        self.window.ui.nodes['editor.app.log.btn.reload'] = QPushButton("RELOAD")

        self.window.ui.nodes['editor.app.log.btn.open'].clicked.connect(
            lambda: self.open_external())
        self.window.ui.nodes['editor.app.log.btn.clear'].clicked.connect(
            lambda: self.clear())
        self.window.ui.nodes['editor.app.log.btn.reload'].clicked.connect(
            lambda: self.reload())

        bottom_layout = QHBoxLayout()
        bottom_layout.addWidget(self.window.ui.nodes['editor.app.log.btn.open'])
        bottom_layout.addWidget(self.window.ui.nodes['editor.app.log.btn.clear'])
        bottom_layout.addWidget(self.window.ui.nodes['editor.app.log.btn.reload'])

        self.window.ui.paths[id] = TitleLabel("")

        self.window.ui.nodes['dialog.app.log.label'] = QLabel(self.get_log_path())
        self.window.ui.nodes['dialog.app.log.level'] = QLabel("")
        self.window.ui.nodes['dialog.app.log.level'].setAlignment(Qt.AlignRight)
        self.update_log_level()

        top = QHBoxLayout()
        top.addWidget(self.window.ui.nodes['dialog.app.log.label'])
        top.addWidget(self.window.ui.nodes['dialog.app.log.level'])

        layout = QVBoxLayout()
        layout.addLayout(top)
        layout.addWidget(self.window.ui.editor['app.log'])
        layout.addLayout(bottom_layout)

        self.window.ui.dialog['app.log'] = AppLogDialog(self.window)
        self.window.ui.dialog['app.log'].setLayout(layout)
        self.window.ui.dialog['app.log'].setWindowTitle("app.log")

    def update_log_level(self):
        """Update log level"""
        level = self.window.core.debug.get_log_level_name().upper()
        self.window.ui.nodes['dialog.app.log.level'].setText("Log level: " + level)

    def update(self):
        """Update app log dialog"""
        self.window.ui.nodes['dialog.app.log.label'].setText(self.get_log_path())
        self.update_log_level()
        self.reload(show=False)

    def reload(self, show: bool = True):
        """
        Reload app.log file

        :param show: show dialog
        """
        path = self.get_log_path()

        # load data from log file
        if os.path.exists(path):
            try:
                with open(path, 'rb') as file:
                    content_bytes = file.read()
                    content = content_bytes.decode('utf-8', errors='ignore')
                    self.window.ui.editor['app.log'].setPlainText(content)
            except Exception as e:
                print(e)

        if show:
            self.window.ui.dialog['app.log'].show()

        # scroll to the end
        cursor = self.window.ui.editor['app.log'].textCursor()
        cursor.movePosition(QTextCursor.End)
        self.window.ui.editor['app.log'].setTextCursor(cursor)

    def get_log_path(self) -> str:
        """
        Get log file path

        :return: log path
        """
        return os.path.join(self.window.core.config.get_user_path(), 'app.log')

    def clear(self, force: bool = False):
        """
        Clear app.log file

        :param force: force clear
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='app.log.clear',
                id=-1,
                msg="Clear app.log file?",
            )
            return

        path = self.get_log_path()
        if os.path.exists(path):
            with open(path, 'w', encoding="utf-8") as file:
                file.write('')

        self.reload()

    def open_external(self):
        """Open log file in external editor"""
        path = self.get_log_path()
        if os.path.exists(path):
            self.window.controller.files.open(path)
        else:
            print("Log file not found!")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.08 21:00:00                  #
# ================================================== #

import datetime

from PySide6 import QtWidgets, QtCore, QtGui
from PySide6.QtGui import QAction, QIcon, QColor, QPixmap, QStandardItem
from PySide6.QtWidgets import QMenu

from pygpt_net.ui.widget.lists.base import BaseList
from pygpt_net.utils import trans
import pygpt_net.icons_rc


class ContextList(BaseList):
    def __init__(self, window=None, id=None):
        """
        Context select menu

        :param window: main window
        :param id: input id
        """
        super(ContextList, self).__init__(window)
        self.window = window
        self.id = id
        self.clicked.connect(self.click)
        self.expanded_items = set()
        self.setItemDelegate(ImportantItemDelegate())

    def click(self, index):
        item = self.window.ui.models['ctx.list'].itemFromIndex(index)
        # check for attribute
        if not hasattr(item, 'isFolder'):
            return
        if item.isFolder:
            if self.window.ui.nodes['ctx.list'].isExpanded(index):
                self.expanded_items.discard(item.id)
            else:
                self.expanded_items.add(item.id)
        else:
            self.window.controller.ctx.select_by_id(item.id)

    def expand_group(self, id):
        """
        Expand group

        :param id: group id
        """
        for i in range(self.window.ui.models['ctx.list'].rowCount()):
            item = self.window.ui.models['ctx.list'].item(i)
            if item.id == id:
                index = self.window.ui.models['ctx.list'].indexFromItem(item)
                self.window.ui.nodes['ctx.list'].expand(index)
                self.expanded_items.add(id)

    def dblclick(self, index):
        """
        Double click event

        :param index: index
        """
        pass

    def contextMenuEvent(self, event):
        """
        Context menu event

        :param event: context menu event
        """
        index = self.indexAt(event.pos())
        item = self.window.ui.models['ctx.list'].itemFromIndex(index)
        idx = item.row()
        id = item.id

        # group context menu
        if hasattr(item, 'isFolder') and item.isFolder:
            actions = {}
            actions['new'] = QAction(QIcon(":/icons/add.svg"), trans('action.ctx.new'), self)
            actions['new'].triggered.connect(
                lambda checked=False, id=id: self.window.controller.ctx.new(force=False, group_id=id)
            )
            actions['rename'] = QAction(QIcon(":/icons/edit.svg"), trans('action.rename'), self)
            actions['rename'].triggered.connect(
                lambda checked=False, id=id: self.window.controller.ctx.rename_group(id)
            )
            actions['delete'] = QAction(QIcon(":/icons/delete.svg"), trans('action.group.delete.only'), self)
            actions['delete'].triggered.connect(
                lambda checked=False, id=id: self.window.controller.ctx.delete_group(id)
            )
            actions['delete_all'] = QAction(QIcon(":/icons/delete.svg"), trans('action.group.delete.all'), self)
            actions['delete_all'].triggered.connect(
                lambda checked=False, id=id: self.window.controller.ctx.delete_group_all(id)
            )

            menu = QMenu(self)
            menu.addAction(actions['new'])
            menu.addAction(actions['rename'])
            menu.addAction(actions['delete'])  # delete group
            menu.addAction(actions['delete_all'])  # delete group and all contexts

            if idx >= 0:
                menu.exec_(event.globalPos())

        # children context menu
        else:
            ctx_id = id
            ctx = self.window.core.ctx.get_meta_by_id(id)
            if ctx is None:
                return

            is_important = ctx.important

            actions = {}
            actions['rename'] = QAction(QIcon(":/icons/edit.svg"), trans('action.rename'), self)
            actions['rename'].triggered.connect(
                lambda checked=False, ctx_id=ctx_id: self.action_rename(ctx_id)
            )

            if is_important:
                actions['important'] = QAction(QIcon(":/icons/pin.svg"), trans('action.unpin'), self)
                actions['important'].triggered.connect(
                    lambda checked=False, ctx_id=ctx_id: self.action_unpin(ctx_id)
                )
            else:
                actions['important'] = QAction(QIcon(":/icons/pin.svg"), trans('action.pin'), self)
                actions['important'].triggered.connect(
                    lambda checked=False, ctx_id=ctx_id: self.action_pin(ctx_id)
                )

            actions['duplicate'] = QAction(QIcon(":/icons/copy.svg"), trans('action.duplicate'), self)
            actions['duplicate'].triggered.connect(
                lambda checked=False, ctx_id=ctx_id: self.action_duplicate(ctx_id)
            )

            actions['delete'] = QAction(QIcon(":/icons/delete.svg"), trans('action.delete'), self)
            actions['delete'].triggered.connect(
                lambda checked=False, ctx_id=ctx_id: self.action_delete(ctx_id)
            )

            actions['copy_id'] = QAction(QIcon(":/icons/copy.svg"), trans('action.ctx_copy_id') + " @" + str(id), self)
            actions['copy_id'].triggered.connect(
                lambda checked=False, ctx_id=ctx_id: self.action_copy_id(ctx_id)
            )

            menu = QMenu(self)
            menu.addAction(actions['rename'])
            menu.addAction(actions['duplicate'])
            menu.addAction(actions['important'])
            menu.addAction(actions['delete'])

            # set label menu
            colors = self.window.controller.ui.get_colors()
            set_label_menu = menu.addMenu(trans('calendar.day.label'))
            for status_id, status_info in colors.items():
                name = trans('calendar.day.' + status_info['label'])
                if status_id == 0:
                    name = '-'
                color = status_info['color']
                pixmap = QPixmap(16, 16)
                pixmap.fill(color)
                icon = QIcon(pixmap)
                status_action = QAction(icon, name, self)
                status_action.triggered.connect(
                    lambda checked=False,
                           ctx_id=ctx_id,
                           s_id=status_id: self.window.controller.ctx.set_label(ctx_id, s_id)
                )
                set_label_menu.addAction(status_action)

            idx_menu = QMenu(trans('action.idx'), self)

            # indexes list
            idxs = self.window.core.config.get('llama.idx.list')
            store = self.window.core.idx.get_current_store()  # get current idx store provider
            if len(idxs) > 0:
                for index in idxs:
                    id = index['id']
                    name = index['name'] + " (" + index['id'] + ")"

                    # add to index
                    action = idx_menu.addAction("IDX: " + name)
                    action.setIcon(QIcon(":/icons/search.svg"))
                    action.triggered.connect(
                        lambda checked=False,
                               ctx_id=ctx_id,
                               index=id: self.action_idx(ctx_id, index)
                    )

                    # remove from index
                    if ctx.indexed is not None and ctx.indexed > 0:
                        # get list of indexes in which context is indexed
                        if store in ctx.indexes:
                            store_indexes = ctx.indexes[store]
                            for store_index in store_indexes:
                                action = idx_menu.addAction(trans("action.idx.remove") + ": " + store_index)
                                action.setIcon(QIcon(":/icons/delete.svg"))
                                action.triggered.connect(
                                    lambda checked=False,
                                           store_index=store_index,
                                           ctx_id=ctx_id: self.action_idx_remove(store_index, ctx_id)  # by context meta id
                                )
                menu.addMenu(idx_menu)

            # -----------------------------------------

            # move to group menu
            group_menu = QMenu(trans('action.move_to'), self)
            groups = self.window.core.ctx.get_groups()

            # add group
            action = group_menu.addAction(trans("action.group.new"))
            action.setIcon(QIcon(":/icons/add.svg"))
            action.triggered.connect(
                lambda checked=False, ctx_id=ctx_id: self.window.controller.ctx.new_group(ctx_id)
            )

            # add separator if groups exists
            if len(groups) > 0:
                group_menu.addSeparator()

            # list of groups
            for group_id in groups:
                group = groups[group_id]
                action = group_menu.addAction(group.name)
                action.triggered.connect(
                    lambda checked=False,
                           group_id=group_id,
                           ctx_id=ctx_id: self.window.controller.ctx.move_to_group(ctx_id, group_id)
                )

            # add separator if groups exists
            if len(groups) > 0:
                group_menu.addSeparator()

            # if in group add remove from group
            if ctx.group_id is not None and ctx.group_id > 0:
                group_name = str(ctx.group_id)
                if ctx.group_id in groups:
                    group_name = groups[ctx.group_id].name
                action = group_menu.addAction(trans("action.group.remove") + ": " + group_name)
                action.setIcon(QIcon(":/icons/delete.svg"))
                action.triggered.connect(
                    lambda checked=False,
                           ctx_id=ctx_id: self.window.controller.ctx.remove_from_group(ctx_id)
                )

            menu.addMenu(group_menu)

            menu.addAction(actions['copy_id'])

            # -----------------------------------------

            # show last indexed date if available
            if ctx.indexed is not None and ctx.indexed > 0:
                suffix = ""
                if ctx.updated > ctx.indexed:
                    suffix = " *"
                dt = datetime.datetime.fromtimestamp(ctx.indexed).strftime("%Y-%m-%d %H:%M")
                action = QAction(QIcon(":/icons/clock.svg"), trans('action.ctx.indexed') + ": " + dt + suffix, self)
                action.setEnabled(False)  # disable action, only for info
                menu.addAction(action)

            if idx >= 0:
                self.window.controller.ctx.select_by_id(ctx_id)
                menu.exec_(event.globalPos())

    def action_idx(self, id: int, idx: int):
        """
        Index with llama context action handler

        :param id: context id
        :param idx: index name
        """
        self.window.controller.idx.indexer.index_ctx_meta(id, idx)

    def action_idx_remove(self, idx: str, meta_id: int):
        """
        Remove from index action handler

        :param idx: index id
        :param meta_id: meta id
        """
        self.window.controller.idx.indexer.index_ctx_meta_remove(idx, meta_id)

    def action_rename(self, id):
        """
        Rename action handler

        :param id: context id
        """
        self.window.controller.ctx.rename(id)

    def action_pin(self, id):
        """
        Pin action handler

        :param id: context id
        """
        self.window.controller.ctx.set_important(id, True)

    def action_unpin(self, id):
        """
        Unpin action handler

        :param id: context id
        """
        self.window.controller.ctx.set_important(id, False)

    def action_important(self, id):
        """
        Set as important action handler

        :param id: context id
        """
        self.window.controller.ctx.set_important(id)

    def action_duplicate(self, id):
        """
        Rename duplicate handler

        :param id: context id
        """
        self.window.controller.ctx.common.duplicate(id)

    def action_delete(self, id):
        """
        Delete action handler

        :param id: context id
        """
        self.window.controller.ctx.delete(id)

    def action_copy_id(self, id):
        """
        Copy ID tag action handler

        :param id: context id
        """
        self.window.controller.ctx.common.copy_id(id)


class ImportantItemDelegate(QtWidgets.QStyledItemDelegate):
    """
    Label color delegate

    :param QtWidgets.QStyledItemDelegate: parent class
    """
    def paint(self, painter, option, index):
        if index.parent().isValid():
            option.rect.adjust(15, 0, 0, 0)

        super(ImportantItemDelegate, self).paint(painter, option, index)

        # pin (>= 10)
        if index.data(QtCore.Qt.ItemDataRole.UserRole) and index.data(QtCore.Qt.ItemDataRole.UserRole) > 0:
            label = index.data(QtCore.Qt.ItemDataRole.UserRole)
            painter.save()

            if label >= 10:
                color = self.get_color_for_status(3)
                square_size = 3
                square_margin = 0
                square_rect = QtCore.QRect(
                    option.rect.left() + square_margin,
                    option.rect.top() + 2,
                    square_size,
                    square_size,
                )
                painter.setBrush(color)
                painter.setPen(
                    QtGui.QPen(
                        QtCore.Qt.black,
                        0.5,
                        QtCore.Qt.SolidLine,
                    )
                )
                painter.drawRect(square_rect)

                label = label - 10  # remove pin status

            # label (0-9)
            if label > 0:
                color = self.get_color_for_status(label)
                square_size = 5
                square_margin = 0
                square_rect = QtCore.QRect(
                    option.rect.left() + square_margin,
                    option.rect.center().y() - (square_size / 2) + 2,
                    square_size,
                    square_size,
                )
                painter.setBrush(color)
                painter.setPen(QtCore.Qt.NoPen)
                painter.drawRect(square_rect)

            painter.restore()

    def get_color_for_status(self, status: int) -> QColor:
        """
        Get color for status

        :param status: status id
        :return: color
        """
        statuses = {
            0: {'label': 'label.color.default', 'color': QColor(100, 100, 100)},
            1: {'label': 'label.color.red', 'color': QColor(255, 0, 0)},
            2: {'label': 'label.color.orange', 'color': QColor(255, 165, 0)},
            3: {'label': 'label.color.yellow', 'color': QColor(255, 255, 0)},
            4: {'label': 'label.color.green', 'color': QColor(0, 255, 0)},
            5: {'label': 'label.color.blue', 'color': QColor(0, 0, 255)},
            6: {'label': 'label.color.indigo', 'color': QColor(75, 0, 130)},
            7: {'label': 'label.color.violet', 'color': QColor(238, 130, 238)},
        }
        if status in statuses:
            return statuses[status]['color']
        else:
            return statuses[0]['color']

class GroupItem(QStandardItem):
    def __init__(self, icon, name, id):
        super().__init__(icon, name)
        self.id = id
        self.name = name
        self.isFolder = True

class Item(QStandardItem):
    def __init__(self, name, id):
        super().__init__(name)
        self.id = id
        self.name = name
        self.isFolder = False


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.08 21:00:00                  #
# ================================================== #

import json
import re
from datetime import datetime, timedelta, timezone

from pygpt_net.item.ctx import CtxMeta, CtxItem, CtxGroup
from pygpt_net.utils import unpack_var


def search_by_date_string(search_string: str) -> list:
    """
    Prepare date ranges from search string if @date() syntax is used

    :param search_string: search string
    :return: list of date ranges
    """
    date_pattern = re.compile(r'@date\((\d{4}-\d{2}-\d{2})?(,)?(\d{4}-\d{2}-\d{2})?\)')
    matches = date_pattern.findall(search_string)
    date_ranges = []

    for match in matches:
        start_date_str, sep, end_date_str = match
        if start_date_str and end_date_str:
            # search between dates
            start_ts = datetime.strptime(start_date_str, '%Y-%m-%d').replace(tzinfo=timezone.utc).timestamp()
            end_ts = datetime.strptime(end_date_str, '%Y-%m-%d').replace(tzinfo=timezone.utc).timestamp()
            # Add one day and subtract one second to include the end date entirely
            end_ts += (24 * 60 * 60) - 1
            date_ranges.append((start_ts, end_ts))
        elif start_date_str and sep:
            # search from date to infinity
            start_ts = datetime.strptime(start_date_str, '%Y-%m-%d').replace(tzinfo=timezone.utc).timestamp()
            end_of_day_ts = None
            date_ranges.append((start_ts, end_of_day_ts))
        elif end_date_str and sep:
            # search from beginning of time to date
            end_ts = datetime.strptime(end_date_str, '%Y-%m-%d').replace(tzinfo=timezone.utc).timestamp()
            # Add one day and subtract one second to include the end date entirely
            end_ts += (24 * 60 * 60) - 1
            date_ranges.append((None, end_ts))
        elif start_date_str:
            # search in exact day
            start_ts = datetime.strptime(start_date_str, '%Y-%m-%d').replace(tzinfo=timezone.utc).timestamp()
            # Add one day and subtract one second to include the entire day
            end_of_day_ts = start_ts + (24 * 60 * 60) - 1
            date_ranges.append((start_ts, end_of_day_ts))
        elif end_date_str:
            # search in exact day
            end_ts = datetime.strptime(end_date_str, '%Y-%m-%d').replace(tzinfo=timezone.utc).timestamp()
            # Add one day and subtract one second to include the entire day
            end_ts += (24 * 60 * 60) - 1
            date_ranges.append((0, end_ts))

    return date_ranges


def get_month_start_end_timestamps(year: int, month: int) -> (int, int):
    """
    Get start and end timestamps for given month

    :param year: year
    :param month: month
    :return: start and end timestamps
    """
    start_date = datetime(year, month, 1)
    start_timestamp = int(start_date.timestamp())
    if month == 12:
        end_date = datetime(year+1, 1, 1) - timedelta(seconds=1)
    else:
        end_date = datetime(year, month+1, 1) - timedelta(seconds=1)
    end_timestamp = int(end_date.timestamp())
    return start_timestamp, end_timestamp


def get_year_start_end_timestamps(year: int) -> (int, int):
    """
    Get start and end timestamps for given year

    :param year: year
    :return: start and end timestamps
    """
    start_date = datetime(year, 1, 1)
    start_timestamp = int(start_date.timestamp())
    end_date = datetime(year+1, 1, 1) - timedelta(seconds=1)
    end_timestamp = int(end_date.timestamp())
    return start_timestamp, end_timestamp


def pack_item_value(value: any) -> str:
    """
    Pack item value to JSON

    :param value: Value to pack
    :return: JSON string or value itself
    """
    if isinstance(value, (list, dict)):
        return json.dumps(value)
    return value


def unpack_item_value(value: any) -> any:
    """
    Unpack item value from JSON

    :param value: Value to unpack
    :return: Unpacked value
    """
    if value is None:
        return None
    try:
        return json.loads(value)
    except:
        return value


def unpack_item(item: CtxItem, row: dict) -> CtxItem:
    """
    Unpack context item from DB row

    :param item: Context item (CtxItem)
    :param row: DB row
    :return: context item
    """
    item.id = unpack_var(row['id'], 'int')
    item.meta_id = unpack_var(row['meta_id'], 'int')
    item.external_id = row['external_id']
    item.input = row['input']
    item.output = row['output']
    item.input_name = row['input_name']
    item.output_name = row['output_name']
    item.input_timestamp = unpack_var(row['input_ts'], 'int')
    item.output_timestamp = unpack_var(row['output_ts'], 'int')
    item.mode = row['mode']
    item.model = row['model']
    item.thread = row['thread_id']
    item.msg_id = row['msg_id']
    item.run_id = row['run_id']
    item.cmds = unpack_item_value(row['cmds_json'])
    item.results = unpack_item_value(row['results_json'])
    item.urls = unpack_item_value(row['urls_json'])
    item.images = unpack_item_value(row['images_json'])
    item.files = unpack_item_value(row['files_json'])
    item.attachments = unpack_item_value(row['attachments_json'])
    item.extra = unpack_item_value(row['extra'])
    item.input_tokens = unpack_var(row['input_tokens'], 'int')
    item.output_tokens = unpack_var(row['output_tokens'], 'int')
    item.total_tokens = unpack_var(row['total_tokens'], 'int')
    item.internal = unpack_var(row['is_internal'], 'bool')
    item.doc_ids = unpack_item_value(row['docs_json'])
    return item


def unpack_meta(meta: CtxMeta, row: dict) -> CtxMeta:
    """
    Unpack context meta data from DB row

    :param meta: Context meta (CtxMeta)
    :param row: DB row
    :return: context meta
    """
    meta.id = unpack_var(row['id'], 'int')
    meta.external_id = row['external_id']
    meta.uuid = row['uuid']
    meta.created = unpack_var(row['created_ts'], 'int')
    meta.updated = unpack_var(row['updated_ts'], 'int')
    meta.indexed = unpack_var(row['indexed_ts'], 'int')
    meta.name = row['name']
    meta.mode = row['mode']
    meta.model = row['model']
    meta.last_mode = row['last_mode']
    meta.last_model = row['last_model']
    meta.thread = row['thread_id']
    meta.assistant = row['assistant_id']
    meta.preset = row['preset_id']
    meta.run = row['run_id']
    meta.status = row['status']
    meta.extra = row['extra']
    meta.initialized = unpack_var(row['is_initialized'], 'bool')
    meta.deleted = unpack_var(row['is_deleted'], 'bool')
    meta.important = unpack_var(row['is_important'], 'bool')
    meta.archived = unpack_var(row['is_archived'], 'bool')
    meta.label = unpack_var(row['label'], 'int')
    meta.indexes = unpack_item_value(row['indexes_json'])
    meta.group_id = unpack_var(row['group_id'], 'int')
    return meta


def unpack_group(group: CtxGroup, row: dict) -> CtxGroup:
    """
    Unpack context group data from DB row

    :param group: Context group (CtxGroup)
    :param row: DB row
    :return: context meta
    """
    group.id = unpack_var(row['id'], 'int')
    group.uuid = row['uuid']
    group.created = unpack_var(row['created_ts'], 'int')
    group.updated = unpack_var(row['updated_ts'], 'int')
    group.name = row['name']
    return group



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.13 15:00:00                  #
# ================================================== #

from pygpt_net.controller.chat.common import Common
from pygpt_net.controller.chat.files import Files
from pygpt_net.controller.chat.image import Image
from pygpt_net.controller.chat.input import Input
from pygpt_net.controller.chat.output import Output
from pygpt_net.controller.chat.render import Render
from pygpt_net.controller.chat.text import Text
from pygpt_net.controller.chat.vision import Vision


class Chat:
    def __init__(self, window=None):
        """
        Chat input and output controller

        :param window: Window instance
        """
        self.window = window
        self.common = Common(window)
        self.files = Files(window)
        self.image = Image(window)
        self.input = Input(window)
        self.output = Output(window)
        self.render = Render(window)
        self.text = Text(window)
        self.vision = Vision(window)

    def setup(self):
        """Setup"""
        self.common.setup()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.01 18:00:00                  #
# ================================================== #

import webbrowser


class Info:
    def __init__(self, window=None):
        """
        Info dialogs controller

        :param window: Window instance
        """
        self.window = window

        # prepare info ids
        self.ids = ['about', 'changelog', 'license']
        self.active = {}

        # prepare active
        for id in self.ids:
            self.active[id] = False

    def setup(self):
        pass

    def toggle(self, id: str, width: int = 400, height: int = 400):
        """
        Toggle info dialog

        :param id: dialog to toggle
        :param width: dialog width
        :param height: dialog height
        """
        if id in self.active and self.active[id]:
            self.window.ui.dialogs.close('info.' + id)
            self.active[id] = False
        else:
            self.window.ui.dialogs.open(
                'info.' + id,
                width=width,
                height=height,
            )
            self.active[id] = True

        # update menu
        self.update_menu()

    def goto_website(self):
        """Open project website"""
        webbrowser.open(self.window.meta['website'])

    def goto_docs(self):
        """Open docs"""
        webbrowser.open(self.window.meta['docs'])

    def goto_pypi(self):
        """Open PyPi"""
        webbrowser.open(self.window.meta['pypi'])

    def goto_github(self):
        """Open GitHub page"""
        webbrowser.open(self.window.meta['github'])

    def goto_snap(self):
        """Open Snapcraft page"""
        webbrowser.open(self.window.meta['snap'])

    def goto_update(self):
        """Open update URL"""
        webbrowser.open(self.window.meta['website'])

    def update_menu(self):
        """Update info menu"""
        for id in self.ids:
            if id in self.active and self.active[id]:
                self.window.ui.menu['info.' + id].setChecked(True)
            else:
                self.window.ui.menu['info.' + id].setChecked(False)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.21 01:00:00                  #
# ================================================== #

import json

from pygpt_net.item.attachment import AttachmentItem


class AssistantItem:
    def __init__(self):
        """
        Assistant item
        """
        self.id = None
        self.name = None
        self.description = None
        self.instructions = None
        self.model = None
        self.meta = {}
        self.files = {}  # file IDs (files uploaded to remote storage)
        self.attachments = {}  # local attachments
        self.tools = {
            "code_interpreter": False,
            "retrieval": False,
            "function": [],
        }

    def reset(self):
        """
        Reset assistant
        """
        self.id = None
        self.name = None
        self.description = None
        self.instructions = None
        self.model = None
        self.meta = {}
        self.files = {}
        self.attachments = {}
        self.tools = {
            "code_interpreter": False,
            "retrieval": False,
            "function": [],
        }

    def to_dict(self):
        return {
            "id": self.id,
            "name": self.name,
            "description": self.description,
            "instructions": self.instructions,
            "model": self.model,
            "meta": self.meta,
            "files": self.files,
            "attachments": self.attachments,
            "tool.code_interpreter": self.tools["code_interpreter"],
            "tool.retrieval": self.tools["retrieval"],
            "tool.function": self.tools["function"],
        }

    def clear_functions(self):
        """
        Clear functions
        """
        self.tools['function'] = []

    def clear_tools(self):
        """
        Clear tools
        """
        self.tools = {
            "code_interpreter": False,
            "retrieval": False,
            "function": [],
        }

    def add_function(self, name: str, parameters: str, desc: str):
        """
        Add function to assistant

        :param name: function name
        :param parameters: function parameters (JSON encoded)
        :param desc: function description
        """
        function = {
            'name': name,
            'params': parameters,
            'desc': desc,
        }
        self.tools['function'].append(function)

    def has_functions(self) -> bool:
        """
        Check if assistant has functions

        :return: bool
        """
        return len(self.tools['function']) > 0

    def get_functions(self) -> list:
        """
        Return assistant functions

        :return: functions
        """
        return self.tools['function']

    def has_tool(self, tool: str) -> bool:
        """
        Check if assistant has tool

        :param tool: tool name
        :return: bool
        """
        return tool in self.tools and self.tools[tool] is True

    def has_file(self, id: str) -> bool:
        """
        Check if assistant has file with ID

        :param id: file ID
        """
        return id in self.files

    def add_file(self, id: str):
        """
        Add empty file to assistant

        :param id: file ID
        """
        self.files[id] = {}
        self.files[id]['id'] = id

    def delete_file(self, file_id: str):
        """
        Delete file from assistant

        :param file_id: file ID
        """
        if file_id in self.files:
            self.files.pop(file_id)

    def clear_files(self):
        """
        Clear files
        """
        self.files = {}

    def has_attachment(self, id: str) -> bool:
        """
        Check if assistant has attachment with ID

        :param id: attachment ID
        :return: bool
        """
        return id in self.attachments

    def add_attachment(self, attachment: AttachmentItem):
        """
        Add attachment to assistant

        :param attachment: attachment
        """
        id = attachment.id
        self.attachments[id] = attachment

    def delete_attachment(self, id: str):
        """
        Delete attachment from assistant

        :param id: attachment ID
        """
        if id in self.attachments:
            self.attachments.pop(id)

    def clear_attachments(self):
        """
        Clear attachments
        """
        self.attachments = {}

    def dump(self):
        """
        Dump item to string

        :return: serialized item
        :rtype: str
        """
        try:
            return json.dumps(self.to_dict())
        except Exception as e:
            pass
        return ""

    def __str__(self):
        """To string"""
        return self.dump()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.09 23:00:00                  #
# ================================================== #

from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import QMenu

from pygpt_net.ui.widget.lists.base import BaseList
from pygpt_net.utils import trans
import pygpt_net.icons_rc


class ProfileList(BaseList):
    def __init__(self, window=None, id=None):
        """
        Profile menu (in editor)

        :param window: main window
        :param id: parent id
        """
        super(ProfileList, self).__init__(window)
        self.window = window
        self.id = id

    def click(self, val):
        pass

    def contextMenuEvent(self, event):
        """
        Context menu event

        :param event: context menu event
        """
        actions = {}
        actions['use'] = QAction(QIcon(":/icons/check.svg"), trans('action.use'), self)
        actions['use'].triggered.connect(
            lambda: self.action_use(event))
        actions['edit'] = QAction(QIcon(":/icons/edit.svg"), trans('action.edit'), self)
        actions['edit'].triggered.connect(
            lambda: self.action_edit(event))
        actions['duplicate'] = QAction(QIcon(":/icons/copy.svg"), trans('action.duplicate'), self)
        actions['duplicate'].triggered.connect(
            lambda: self.action_duplicate(event))


        actions['reset'] = QAction(QIcon(":/icons/close.svg"), trans('action.reset'), self)
        actions['reset'].triggered.connect(
            lambda: self.action_reset(event))
        actions['delete'] = QAction(QIcon(":/icons/delete.svg"), trans('action.profile.delete'), self)
        actions['delete'].triggered.connect(
            lambda: self.action_delete(event))
        actions['delete_all'] = QAction(QIcon(":/icons/delete.svg"), trans('action.profile.delete_all'), self)
        actions['delete_all'].triggered.connect(
            lambda: self.action_delete_all(event))

        menu = QMenu(self)
        menu.addAction(actions['edit'])
        menu.addAction(actions['use'])
        menu.addAction(actions['duplicate'])
        menu.addAction(actions['reset'])
        menu.addAction(actions['delete'])
        menu.addAction(actions['delete_all'])

        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            menu.exec_(event.globalPos())

    def action_use(self, event):
        """
        Use action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.settings.profile.select_by_idx(idx)

    def action_duplicate(self, event):
        """
        Duplicate action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.settings.profile.duplicate_by_idx(idx)

    def action_reset(self, event):
        """
        Reset action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.settings.profile.reset_by_idx(idx)

    def action_delete(self, event):
        """
        Delete action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.settings.profile.delete_by_idx(idx)

    def action_delete_all(self, event):
        """
        Delete all action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.settings.profile.delete_all_by_idx(idx)

    def action_edit(self, event):
        """
        Edit action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.settings.profile.edit_by_idx(idx)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.08 03:00:00                  #
# ================================================== #

from PySide6.QtCore import QRegularExpression
from PySide6.QtGui import QTextCursor, Qt


class Finder:
    def __init__(self, window=None):
        """
        Finder controller

        :param window: Window instance
        """
        self.window = window
        self.original_html = {}
        self.id = None
        self.locked = False
        self.opened = False
        self.last_search = {}
        self.matches = {}
        self.current_match_index = {}

    def set_active(self, id):
        """
        Set current active parent id

        :param id: parent id
        """
        self.id = id
        current_search = self.get_search_string()
        if self.id in self.last_search:
            if current_search != self.last_search[self.id]:
                self.clear_search(self.id)
                self.find(current_search)
        else:
            if current_search is not None and len(current_search) > 0:
                self.clear_search(self.id)
                self.find(current_search)
        self.update_counter()

    def clear_search(self, id):
        """
        Clear search state for a specific id

        :param id: parent id
        """
        self.last_search[id] = None
        self.matches[id] = []
        self.current_match_index[id] = -1
        self.update_counter()

    def get_search_string(self):
        """
        Get last search string

        :return: search string
        """
        return self.window.ui.nodes['dialog.find.input'].text()

    def open(self, id):
        """
        Open finder dialog

        :param id: parent id
        """
        if self.opened:
            return
        self.set_active(id)
        self.prepare(clear=False)
        self.window.ui.dialog['find'].show()
        self.opened = True
        self.window.ui.nodes['dialog.find.input'].setFocus()
        current = self.get_search_string()
        if current is not None and len(current) > 0:
            self.window.controller.finder.find(current)

    def close(self, reset: bool = True):
        """
        Close finder dialog

        :param reset: Reset highlights
        """
        if reset:
            self.reset()
        self.window.ui.dialog['find'].hide()
        self.opened = False

    def clear(self, id: str, restore: bool = False, to_end: bool = True):
        """
        Clear current parent highlights

        :param id: parent id
        :param restore: Restore original HTML
        :param to_end: Move cursor to the end
        """
        if id in self.original_html:
            if restore:
                curr_cursor = self.get_current_parent().textCursor()
                self.get_current_parent().setHtml(self.original_html[id])
                if not to_end:
                    self.get_current_parent().setTextCursor(curr_cursor)
            del self.original_html[id]
            self.clear_search(id)
            if to_end:
                self.get_current_parent().moveCursor(QTextCursor.End)
        self.update_counter()

    def clear_input(self):
        """Clear input text"""
        self.window.ui.nodes['dialog.find.input'].clear()
        self.find("")

    def get_current_parent(self):
        """
        Get current parent QTextEdit / QTextBrowser

        :return: QTextEdit / QTextBrowser
        """
        if self.id is not None:
            if self.id == "chat_output":
                return self.window.ui.nodes['output']
            elif self.id.startswith("notepad_"):
                idx = int(self.id.split("_")[1])
                return self.window.ui.notepad[idx].textarea

    def get_original_html(self) -> str:
        """
        Get original HTML for the current parent

        :return: original HTML
        """
        if self.id is not None:
            if self.id in self.original_html:
                return self.original_html[self.id]

    def update_counter(self):
        """Update counter for current parent"""
        if not self.opened:
            return
        if self.id is not None:
            current = self.current_match_index[self.id] + 1 if self.current_match_index.get(self.id, -1) >= 0 else 0
            all = len(self.matches.get(self.id, []))
            self.window.ui.nodes['dialog.find.counter'].setText("{}/{}".format(current, all))
        else:
            self.window.ui.nodes['dialog.find.counter'].setText("0/0")

    def prepare(self, clear: bool = True, to_end: bool = True):
        """
        Prepare the finder for a new search

        :param clear: clear search state
        :param to_end: move cursor to the end
        """
        if self.id not in self.matches:
            self.matches[self.id] = []
        if self.id not in self.current_match_index:
            self.current_match_index[self.id] = -1

        # reset highlights and clear search state
        if clear:
            self.reset(to_end)
            self.clear_search(self.id)
            self.update_current()

    def update_current(self):
        """Update the current parent original HTML"""
        self.original_html[self.id] = self.get_current_parent().toHtml()

    def find(self, text: str):
        """
        Find and highlight all occurrences of the text

        :param text: text to find
        """
        if self.id not in self.last_search or text != self.last_search.get(self.id, None):
            self.prepare(clear=True, to_end=False)

        if text == self.last_search.get(self.id, None):
            self.matches[self.id] = []

        if text is None or len(text) == 0:
            self.last_search[self.id] = None
            self.matches[self.id] = []
            return

        self.last_search[self.id] = text
        parent = self.get_current_parent()

        # make regex safe search string:
        text = QRegularExpression.escape(text)

        # use QTextCursor for searching
        cursor = QTextCursor(parent.document())
        regex = QRegularExpression(text, QRegularExpression.CaseInsensitiveOption)
        plain_text = parent.toPlainText()
        index = regex.match(plain_text)

        # highlight and store all matches
        while index.hasMatch():
            pos = index.capturedStart()
            length = index.capturedLength()
            self.matches[self.id].append((pos, length))  # save position and length of match
            cursor.setPosition(pos)
            cursor.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, length)
            self.highlight(cursor)
            index = regex.match(plain_text, pos + length)

        self.update_counter()
        if self.matches[self.id] and self.id in self.current_match_index:
            idx = self.current_match_index[self.id]
            if idx < 0:
                self.find_next()

    def find_next(self):
        """Find the next occurrence relative to the current match index for the active id"""
        if not self.matches.get(self.id, []):
            return

        self.current_match_index[self.id] = (self.current_match_index[self.id] + 1) % len(self.matches[self.id])
        self.scroll(self.id, self.current_match_index[self.id])
        self.update_counter()

    def find_prev(self):
        """Find the previous occurrence relative to the current match index for the active id"""
        if not self.matches.get(self.id, []):
            return

        self.current_match_index[self.id] = (self.current_match_index[self.id] - 1) % len(self.matches[self.id])
        self.scroll(self.id, self.current_match_index[self.id])
        self.update_counter()

    def scroll(self, id: str, match_index: int):
        """
        Scroll to the match given by match_index for the specified id

        :param id: str
        :param match_index: int
        """
        parent = self.get_current_parent()
        if match_index < 0 or match_index >= len(self.matches[id]):
            return

        pos, _ = self.matches[id][match_index]
        cursor = QTextCursor(parent.document())
        cursor.setPosition(pos)
        parent.setTextCursor(cursor)

    def highlight(self, cursor):
        """
        Highlight the selection of the given cursor

        :param cursor: QTextCursor
        """
        fmt = cursor.charFormat()
        fmt.setForeground(Qt.black)
        fmt.setBackground(Qt.yellow)
        cursor.setCharFormat(fmt)

    def get_current_scroll_position(self):
        """
        Get current scroll position

        :return: scroll position
        """
        return self.get_current_parent().verticalScrollBar().value()

    def set_current_scroll_position(self, value):
        """
        Set current scroll position

        :param value: scroll position
        """
        self.get_current_parent().verticalScrollBar().setValue(value)

    def reset(self, to_end: bool = True):
        """
        Reset highlights

        :param to_end: move cursor to the end
        """
        self.locked = True
        html = self.get_original_html()
        if html is None:
            self.locked = False
            return
        now_html = self.get_current_parent().toHtml()
        if now_html == html:
            self.locked = False
            return

        pos = self.get_current_scroll_position()
        self.get_current_parent().setHtml(self.get_original_html())
        self.set_current_scroll_position(pos)

        # move cursor to end
        if to_end:
            cursor = QTextCursor(self.get_current_parent().document())
            cursor.movePosition(QTextCursor.End)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.15 11:00:00                  #
# ================================================== #

import datetime

from PySide6.QtCore import Qt, QPoint
from PySide6.QtGui import QImage, QPainter, QPen, QAction, QIcon
from PySide6.QtWidgets import QMenu, QWidget, QFileDialog, QMessageBox, QApplication

from pygpt_net.utils import trans
import pygpt_net.icons_rc


class PainterWidget(QWidget):
    def __init__(self, window=None):
        super().__init__(window)
        self.window = window
        self.image = QImage(self.size(), QImage.Format_RGB32)
        self.image.fill(Qt.white)
        self.drawing = False
        self.brushSize = 3
        self.brushColor = Qt.black
        self.lastPoint = QPoint()
        self.originalImage = None
        self.undoStack = []
        self.redoStack = []
        self.undoLimit = 10
        self.setFocusPolicy(Qt.StrongFocus)
        self.setFocus()

    def contextMenuEvent(self, event):
        """
        Context menu event

        :param event: Event
        """
        actions = {}
        actions['undo'] = QAction(QIcon(":/icons/undo.svg"), trans('action.undo'), self)
        actions['undo'].triggered.connect(
            lambda: self.undo())
        if self.has_undo():
            actions['undo'].setEnabled(True)
        else:
            actions['undo'].setEnabled(False)

        actions['redo'] = QAction(QIcon(":/icons/redo.svg"), trans('action.redo'), self)
        actions['redo'].triggered.connect(
            lambda: self.redo())
        if self.has_redo():
            actions['redo'].setEnabled(True)
        else:
            actions['redo'].setEnabled(False)

        actions['open'] = QAction(QIcon(":/icons/folder_filled.svg"), trans('action.open'), self)
        actions['open'].triggered.connect(
            lambda: self.action_open())
        actions['capture'] = QAction(QIcon(":/icons/fullscreen.svg"), trans('painter.btn.capture'), self)
        actions['capture'].triggered.connect(
            lambda: self.action_capture())
        actions['save'] = QAction(QIcon(":/icons/save.svg"), trans('img.action.save'), self)
        actions['save'].triggered.connect(
            lambda: self.action_save())
        actions['clear'] = QAction(QIcon(":/icons/close.svg"), trans('painter.btn.clear'), self)
        actions['clear'].triggered.connect(
            lambda: self.action_clear())

        menu = QMenu(self)
        menu.addAction(actions['undo'])
        menu.addAction(actions['redo'])
        menu.addAction(actions['open'])
        menu.addAction(actions['capture'])
        menu.addAction(actions['save'])
        menu.addAction(actions['clear'])
        menu.exec_(event.globalPos())

    def action_open(self):
        """Open the image"""
        path, _ = QFileDialog.getOpenFileName(
            self,
            "Open Image",
            "",
            "Images (*.png *.jpg *.jpeg)",
        )
        if path:
            self.open_image(path)

    def action_capture(self):
        """Capture the image"""
        self.saveForUndo()
        self.window.controller.painter.capture.use()

    def action_save(self):
        """Save image to file"""
        name = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S") + ".png"
        path, _ = QFileDialog.getSaveFileName(
            self,
            "Save Image",
            name,
            "PNG(*.png);;JPEG(*.jpg *.jpeg);;All Files(*.*) ",
        )
        if path:
            self.image.save(path)

    def action_clear(self):
        """Clear the image"""
        self.saveForUndo()
        self.clear_image()
        self.originalImage = self.image

    def open_image(self, path):
        """
        Open the image

        :param path: Path to image
        """
        self.saveForUndo()
        img = QImage(path)
        if img.isNull():
            QMessageBox.information(self, "Image Loader", "Cannot load file.")
            return
        self.scale_to_fit(img)

    def set_image(self, image):
        """
        Set image

        :param image: Image
        """
        self.saveForUndo()
        self.scale_to_fit(image)
        self.originalImage = self.image
        self.update()

    def scale_to_fit(self, image):
        """
        Scale image to fit the widget

        :param image: Image
        """
        if image.width() == self.width():
            new = image
        else:
            # to fit the width
            if image.width() > image.height():
                width = self.width()
                height = (image.height() * self.width()) / image.width()
                new = image.scaled(
                    width,
                    int(height),
                    Qt.KeepAspectRatioByExpanding,
                    Qt.SmoothTransformation,
                )
            else:
                height = self.height()
                width = (image.width() * self.height()) / image.height()
                new = image.scaled(
                    int(width),
                    height,
                    Qt.KeepAspectRatioByExpanding,
                    Qt.SmoothTransformation,
                )

        self.image = QImage(
            self.width(),
            self.height(),
            QImage.Format_RGB32,
        )
        self.image.fill(Qt.white)
        painter = QPainter(self.image)
        painter.drawImage(0, 0, new)
        painter.end()
        self.update()
        self.originalImage = self.image

    def saveForUndo(self):
        """Save current state for undo"""
        if len(self.undoStack) >= self.undoLimit:
            self.undoStack.pop(0)
        self.undoStack.append(self.image.copy())
        self.redoStack.clear()  # clear redo on new action

    def undo(self):
        """Undo the last action"""
        if self.undoStack:
            self.redoStack.append(self.image.copy())
            self.image = self.undoStack.pop()
            self.update()

    def redo(self):
        """Redo the last undo action"""
        if self.redoStack:
            self.undoStack.append(self.image.copy())
            self.image = self.redoStack.pop()
            self.update()

    def has_undo(self) -> bool:
        """Check if undo is available"""
        return bool(self.undoStack)

    def has_redo(self) -> bool:
        """Check if redo is available"""
        return bool(self.redoStack)

    def set_brush_color(self, color):
        """
        Set the brush color

        :param color: Color
        """
        self.brushColor = color

    def set_brush_size(self, size):
        """
        Set the brush size

        :param size: Brush size
        """
        self.brushSize = size

    def clear_image(self):
        """Clear the image"""
        self.saveForUndo()
        self.image.fill(Qt.white)
        self.update()

    def mousePressEvent(self, event):
        """
        Mouse press event

        :param event: Event
        """
        if event.button() == Qt.LeftButton:
            self.drawing = True
            self.lastPoint = event.pos()
            self.saveForUndo()
            painter = QPainter(self.image)
            painter.setPen(
                QPen(
                    self.brushColor,
                    self.brushSize,
                    Qt.SolidLine,
                    Qt.RoundCap,
                    Qt.RoundJoin,
                )
            )
            painter.drawPoint(self.lastPoint)
            self.update()

    def mouseMoveEvent(self, event):
        """
        Mouse move event

        :param event: Event
        """
        if (event.buttons() & Qt.LeftButton) and self.drawing:
            painter = QPainter(self.image)
            painter.setPen(
                QPen(
                    self.brushColor,
                    self.brushSize,
                    Qt.SolidLine,
                    Qt.RoundCap,
                    Qt.RoundJoin,
                )
            )
            painter.drawLine(self.lastPoint, event.pos())
            self.lastPoint = event.pos()
            self.update()

    def mouseReleaseEvent(self, event):
        """
        Mouse release event

        :param event: Event
        """
        if event.button() == Qt.LeftButton or event.button() == Qt.RightButton:
            self.drawing = False

    def keyPressEvent(self, event):
        """
        Key press event to handle undo action

        :param event: Event
        """
        if event.key() == Qt.Key_Z and QApplication.keyboardModifiers() == Qt.ControlModifier:
            self.undo()

    def paintEvent(self, event):
        """
        Paint event (draw)

        :param event: Event
        """
        painter = QPainter(self)
        painter.drawImage(self.rect(), self.image, self.image.rect())
        self.originalImage = self.image

    def resizeEvent(self, event):
        """
        Update coords on resize
        
        :param event: Event
        """
        if self.image.size() != self.size():
            new = QImage(self.size(), QImage.Format_RGB32)
            new.fill(Qt.white)
            painter = QPainter(new)
            painter.drawImage(QPoint(0, 0), self.image)
            self.image = new



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.08 03:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDialog, QHBoxLayout, QVBoxLayout, QPushButton

from pygpt_net.ui.widget.element.labels import HelpLabel
from pygpt_net.ui.widget.textarea.find import FindInput
from pygpt_net.utils import trans


class FindDialog(QDialog):
    def __init__(self, window=None, id=None):
        """
        Find dialog

        :param window: main window
        :param id: info window id
        """
        super(FindDialog, self).__init__(window)
        self.window = window
        self.id = id
        self.current = None
        self.window.ui.nodes['dialog.find.input'] = FindInput(window, id)
        self.window.ui.nodes['dialog.find.input'].setMinimumWidth(400)

        self.window.ui.nodes['dialog.find.input'].textChanged.connect(
            lambda: self.window.controller.finder.find(
                self.window.ui.nodes['dialog.find.input'].text()
            ),
        )

        self.window.ui.nodes['dialog.find.btn.clear'] = QPushButton(trans('dialog.find.btn.clear'))
        self.window.ui.nodes['dialog.find.btn.clear'].clicked.connect(
            lambda: self.window.controller.finder.clear_input(),
        )
        self.window.ui.nodes['dialog.find.btn.find_prev'] = QPushButton(trans('dialog.find.btn.find_prev'))
        self.window.ui.nodes['dialog.find.btn.find_prev'].clicked.connect(
            lambda: self.window.controller.finder.find_prev(),
        )
        self.window.ui.nodes['dialog.find.btn.find_next'] = QPushButton(trans('dialog.find.btn.find_next'))
        self.window.ui.nodes['dialog.find.btn.find_next'].clicked.connect(
            lambda: self.window.controller.finder.find_next(),
        )

        bottom = QHBoxLayout()
        bottom.addWidget(self.window.ui.nodes['dialog.find.btn.clear'])
        bottom.addWidget(self.window.ui.nodes['dialog.find.btn.find_prev'])
        bottom.addWidget(self.window.ui.nodes['dialog.find.btn.find_next'])

        self.window.ui.nodes['dialog.find.counter'] = HelpLabel('0/0')
        self.window.ui.nodes['dialog.find.counter'].setAlignment(Qt.AlignCenter)
        self.window.ui.nodes['dialog.find.counter'].setMinimumWidth(70)
        self.window.ui.nodes['dialog.find.counter'].setMaximumWidth(70)

        input_layout = QHBoxLayout()
        input_layout.addWidget(self.window.ui.nodes['dialog.find.input'])
        input_layout.addWidget(self.window.ui.nodes['dialog.find.counter'])

        layout = QVBoxLayout()
        layout.addLayout(input_layout)
        layout.addLayout(bottom)

        self.setLayout(layout)

    def closeEvent(self, event):
        self.window.controller.finder.close(reset=False)
        event.accept()

    def keyPressEvent(self, event):
        """
        Key press event

        :param event: key press event
        """
        if event.key() == Qt.Key_Escape:
            self.window.controller.finder.close(reset=False)
            self.close()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.26 15:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import QMenuBar, QVBoxLayout, QHBoxLayout, QSizePolicy

from pygpt_net.ui.widget.dialog.base import BaseDialog
from pygpt_net.ui.widget.image.display import ImageLabel
from pygpt_net.utils import trans

class DialogSpawner:
    def __init__(self, window=None):
        """
        Image viewer dialog spawner

        :param window: Window instance
        """
        self.window = window
        self.path = None
        self.id = 'image_preview'

    def setup(self, id: str = None) -> BaseDialog:
        """
        Setup image viewer dialog

        :param id: dialog id
        :return: BaseDialog instance
        """
        source = ImageLabel(self.window, self.path)
        source.setVisible(False)
        pixmap = ImageLabel(self.window, self.path)
        pixmap.setSizePolicy(QSizePolicy(QSizePolicy.Ignored, QSizePolicy.Ignored))

        row = QHBoxLayout()
        row.addWidget(pixmap)

        layout = QVBoxLayout()
        layout.addLayout(row)

        dialog = ImageViewerDialog(self.window, self.id)
        dialog.disable_geometry_store = True  # disable geometry store
        dialog.id = id
        dialog.append_layout(layout)
        dialog.setLayout(layout)
        dialog.source = source
        dialog.pixmap = pixmap

        return dialog


class ImageViewerDialog(BaseDialog):
    def __init__(self, window=None, id=None):
        """
        Image viewer dialog

        :param window: main window
        :param id: info window id
        """
        super(ImageViewerDialog, self).__init__(window, id)
        self.window = window
        self.id = id
        self.disable_geometry_store = False
        self.menu_bar = None
        self.file_menu = None
        self.actions = {}
        self.resizeEvent = self.onResizeEvent  # resize event to adjust the pixmap
        self.source = None
        self.pixmap = None

    def append_layout(self, layout):
        """
        Update layout

        :param layout: layout
        """
        layout.setMenuBar(self.setup_menu())  # attach menu
        self.setLayout(layout)

    def onResizeEvent(self, event):
        """
        Resize event to adjust the pixmap on window resizing

        :param event: resize event
        """
        if self.source.pixmap() and not self.source.pixmap().isNull():
            self.pixmap.setPixmap(
                self.source.pixmap().scaled(
                    self.pixmap.size(),
                    Qt.KeepAspectRatio,
                    Qt.SmoothTransformation
                )
            )
        super(ImageViewerDialog, self).resizeEvent(event)

    def setup_menu(self) -> QMenuBar:
        """
        Setup dialog menu

        :return: QMenuBar
        """
        self.menu_bar = QMenuBar()
        self.file_menu = self.menu_bar.addMenu(trans("menu.file"))

        # new
        self.actions["new"] = QAction(QIcon(":/icons/add.svg"), trans("action.new"))
        self.actions["new"].triggered.connect(
            lambda: self.window.tools.get("viewer").new()
        )

        # open
        self.actions["open"] = QAction(QIcon(":/icons/folder.svg"), trans("action.open"))
        self.actions["open"].triggered.connect(
            lambda: self.window.tools.get("viewer").open_file(self.id, auto_close=True)
        )

        # open (new window)
        self.actions["open_new"] = QAction(QIcon(":/icons/folder.svg"), trans("action.open_new_window"))
        self.actions["open_new"].triggered.connect(
            lambda: self.window.tools.get("viewer").open_file(self.id, auto_close=False)
        )

        # save as
        self.actions["save_as"] = QAction(QIcon(":/icons/save.svg"), trans("action.save_as"))
        self.actions["save_as"].triggered.connect(
            lambda: self.window.tools.get("viewer").save_by_id(self.id)
        )

        # exit
        self.actions["exit"] = QAction(QIcon(":/icons/logout.svg"), trans("menu.file.exit"))
        self.actions["exit"].triggered.connect(
            lambda: self.window.tools.get("viewer").close_preview(self.id)
        )

        # add actions
        self.file_menu.addAction(self.actions["new"])
        self.file_menu.addAction(self.actions["open"])
        self.file_menu.addAction(self.actions["open_new"])
        self.file_menu.addAction(self.actions["save_as"])
        self.file_menu.addAction(self.actions["exit"])

        return self.menu_bar
